#!/bin/sh
#----------------------------------------------------------------------------------------
# /[data/packages|var]/motion/motion_db_backup_restore.sh - backup / restore of database
#
# Copyright (c) 2011 The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:     2011-10-17  jed
# Last Update:  $Id: motion_db_backup_restore.sh 29008 2011-10-18 11:12:51Z jed $
#
# Usage:        motion_db_backup_restore.sh backup|restore [dump-file-name]
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------------------

# read eislib etc.
. /var/install/include/check-eisfair-version
. /var/install/include/eistime

#exec 2>/tmp/motion-backup-restore-$$.log
#set -x

# set platform specific parameters
case ${EISFAIR_SYSTEM} in
    eisfair-1)
        # eisfair-1
        motionconfig_path=/var/motion
        ;;
    *)
        # default eisfair-2
        motionconfig_path=/data/packages/motion
        ;;
esac

motionfile=/etc/config.d/motion

MOTION_SQL_HOST='localhost'
MOTION_SQL_DATABASE='motiondb'
MOTION_SQL_TABLE='security'

action=$1
fname=$2

if [ "${action}" != "" ]
then
    # dump file name
    if [ "${fname}" != "" ]
    then
        dump_file="${fname}"
    else
        dump_file=${motionconfig_path}/motion-db-dump-${EISDATE}.sql
    fi

    ### load motion configuration ###
    if [ -f ${motionfile} ]
    then
        # load configuration
        . ${motionfile}

        case ${action} in
            backup)
                # backup database
                /usr/local/mysql/bin/mysqldump -h${MOTION_SQL_HOST} -u${MOTION_SQL_USER} -p${MOTION_SQL_PASS} ${MOTION_SQL_DATABASE} ${MOTION_SQL_TABLE} > ${dump_file}
                if [ $? -eq 0 ]
                then
                    echo "motion database dumped to file '${dump_file}'."
                fi
                ;;

            restore)
                # restore database
                if [ -f ${dump_file} ]
                then
                    /var/install/bin/ask "Please enter the MySQL root password" "" "*" > /tmp/ask.$$
                    rc=$?
                    root_pass=`cat /tmp/ask.$$`
                    rm -f /tmp/ask.$$
                    if [ ${rc} = 255 ]
                    then
                        rm ${tmpfile}
                        exit 1
                    fi

                    /usr/local/mysql/bin/mysql -h${MOTION_SQL_HOST} -uroot -p${root_pass} -D${MOTION_SQL_DATABASE} < ${dump_file}

                    if [ $? -eq 0 ]
                    then
                        echo "motion database restored from file '${dump_file}'."
                    fi
                else
                    echo "database dump file '${dump_file}' not found."
                fi
                ;;
        esac
    else
        echo "Error: file '${motionfile}' not found!"
    fi
else
    # error
    echo "Usage: $0 backup|restore"
fi
