#!/bin/sh
#----------------------------------------------------------------------------
# /var/install/config.d/motion.sh - configuration generator script for motion daemon
#
# Copyright (c) 2001-2012 The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:     2002-04-28  fm
# Last Update:  $Id: motion.sh 31918 2012-10-20 16:02:02Z jed $
#
# Usage: motion.sh [create-sql-db|force-cgi-update]
#
#                  create-sql-db      - create sql database and table
#                  delete-sql-db      - delete sql table and database
#                  force-cgi-update   - force update of web files
#  
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------------------

# read eislib etc.
. /var/install/include/eislib
. /var/install/include/jedlib
. /var/install/include/check-eisfair-version

#exec 2>/var/install/config.d/motion-trace-$$.log
#set -x
#testroot=/soft/jedmotion                                            # only for testing
 testroot=""

pgmname=`basename $0`

# set platform specific parameters
case ${EISFAIR_SYSTEM} in
    eisfair-1)
        # eisfair-1
        def_doc_root=/var/www/htdocs
        motionconfig_path=${testroot}/var/motion
        motion_apache_user="wwwrun"
        motion_apache_group="nogroup"
        ;;
    *)
        # default eisfair-2
        def_doc_root=/data/www/htdocs
        motionconfig_path=${testroot}/data/packages/motion
        motion_apache_user="www-data"
        motion_apache_group="www-data"
        ;;
esac

### set path names ###
start_path=${testroot}/etc/init.d
bin_path=${testroot}/usr/local/bin
systemlog_path=${testroot}/var/log
systemrun_path=${testroot}/var/run
install_bin_path=${testroot}/var/install/bin
install_menu_path=${testroot}/var/install/menu

### set file names ###
apache2file=${testroot}/etc/config.d/apache2
environmentfile=${testroot}/etc/config.d/environment
usbfile=${testroot}/etc/config.d/usb
motionfile=${testroot}/etc/config.d/motion
mysqlfile=${testroot}/etc/config.d/mysql
passwdfile=${testroot}/etc/passwd
groupfile=${testroot}/etc/group
pidfile=${systemrun_path}/motion.pid
configlog_file=${motionconfig_path}/log/motion-configlog
motion_configfile=${testroot}/var/install/config.d/motion.sh
generate_motionconf=${motionconfig_path}/motion.conf
generate_threadconf=${motionconfig_path}/motion-thread
generate_motioncgiconf=${motionconfig_path}/motion-cgi.conf
generate_motionwgetrc=${motionconfig_path}/.motion-wgetrc

MOTION_SQL_HOST='localhost'
MOTION_SQL_DATABASE='motiondb'
MOTION_SQL_TABLE='security'

### other parameters ###
motion_version="v0.91.0-99"

### load configuration ###
. ${motionfile}
chmod 600 ${motionfile}

### load environment configuration for HTTP_PROXY ###
. ${environmentfile}

### load apache2 configuration ###
if [ -f ${apache2file} ]
then
    webtmp=/tmp/motionroot.$$

    grep "^APACHE2_DOCUMENT_ROOT" ${apache2file} > ${webtmp}
    . ${webtmp}
    rm -f ${webtmp}
else
    APACHE2_DOCUMENT_ROOT=${def_doc_root}
fi

#----------------------------------------------------------------------------------------
# set parameter to 'no'
# input: $1 - name of parameter
#----------------------------------------------------------------------------------------
set_parameter_no()
{
    # uppercase conversion of parameter name
    _spn_pname="`echo ${1} | tr [:lower:] [:upper:]`"

    # when no edit, set start parameter to 'no'
    sed "s/^${_spn_pname}=['\"]yes['\"]/${_spn_pname}='no' /" ${motionfile} >/tmp/CONFIG_FILE_TMP
    cp /tmp/CONFIG_FILE_TMP ${motionfile}
    rm -f /tmp/CONFIG_FILE_TMP
}

#----------------------------------------------------------------------------------------
# ask for sql root password
#----------------------------------------------------------------------------------------
get_sql_root_password()
{
    if [ "${root_pass}" = "" ]
    then
        /var/install/bin/ask "Please enter the MySQL root password" "" "*" > /tmp/ask.$$
        rc=$?
        root_pass=`cat /tmp/ask.$$`
        rm -f /tmp/ask.$$
        if [ ${rc} = 255 ]
        then
            rm ${tmpfile}
            exit 1
        fi
    fi
}

#----------------------------------------------------------------------------------------
# create SQL database and table
#
# $1 - force or ''
#----------------------------------------------------------------------------------------
create_sql_db_and_table()
{
    if [ "$1" = "force" ]
    then
        force=1
    else
        force=0
    fi

    if [ "${MOTION_SQL_ENABLED}" = "yes" ]
    then
        # create sql command file
        root_pass=''
        sql_cmd_file=/var/run/motion-cmd-$$
        for step in 1 2 3 4 5
        do
            # delete sql command file
            rm -f ${sql_cmd_file}

            case ${step} in
                1)
                    step_name="checking sql database user"
                    mecho -n "${step_name} ..."
                    /usr/local/mysql/bin/mysql -h${MOTION_SQL_HOST} -u${MOTION_SQL_USER} -p${MOTION_SQL_PASS} -e"quit" 2>/dev/null

                    if [ $? -ne 0 -o ${force} -eq 1 ]
                    then
                        # sql user doesn't exist, go on...
                        echo
                        get_sql_root_password

                        mecho "${step_name}..."

                        {
                            if [ ${force} -eq 1 ]
                            then
                                echo "DROP USER '${MOTION_SQL_USER}'@'${MOTION_SQL_HOST}';"
                            fi

                            echo "CREATE USER '${MOTION_SQL_USER}'@'${MOTION_SQL_HOST}' IDENTIFIED BY '${MOTION_SQL_PASS}';"
                        } > ${sql_cmd_file}
                    else
                        mecho --info "granted."
                    fi
                    ;;

                2)
                    step_name="checking sql database"
                    mecho -n "${step_name} ..."
                    db_exists=`/usr/local/mysql/bin/mysql -h${MOTION_SQL_HOST} -u${MOTION_SQL_USER} -p${MOTION_SQL_PASS} -e"SHOW DATABASES" | grep -c "${MOTION_SQL_DATABASE}$"`
                    if [ ${db_exists} -eq 0 -o ${force} -eq 1 ]
                    then
                        # sql database doesn't exist or not access rights have been granted, go on...
                        echo
                        get_sql_root_password

                        {
                            if [ ${force} -eq 1 ]
                            then
                                echo "DROP DATABASE ${MOTION_SQL_DATABASE};"
                            fi

                            echo "CREATE DATABASE IF NOT EXISTS ${MOTION_SQL_DATABASE};"
                        } > ${sql_cmd_file}
                    else
                        mecho --info "found."
                    fi
                    ;;

                3)
                    step_name="granting sql database access"
                    mecho -n "${step_name} ..."
                    /usr/local/mysql/bin/mysql -h${MOTION_SQL_HOST} -u${MOTION_SQL_USER} -p${MOTION_SQL_PASS} -e"SHOW GRANTS FOR '${MOTION_SQL_USER}'@'${MOTION_SQL_HOST}'" | grep -q -E "GRANT ALL PRIVILEGES ON .*${MOTION_SQL_DATABASE}.*\.\* TO .*${MOTION_SQL_USER}.*@.*${MOTION_SQL_HOST}" 2> /dev/null

                    if [ $? -ne 0 -o ${force} -eq 1 ]
                    then
                        echo
                        get_sql_root_password
 
                        {
                            echo "GRANT ALL ON ${MOTION_SQL_DATABASE}.* TO '${MOTION_SQL_USER}'@'${MOTION_SQL_HOST}' IDENTIFIED BY '${MOTION_SQL_PASS}';"
                        } > ${sql_cmd_file}
                    else
                        mecho --info "done."
                    fi
                    ;;

                4)
                    step_name="checking sql database table"
                    mecho -n "${step_name} ..."
                    table_exists=`/usr/local/mysql/bin/mysql -h${MOTION_SQL_HOST} -u${MOTION_SQL_USER} -p${MOTION_SQL_PASS} -D${MOTION_SQL_DATABASE} -e"SHOW TABLES" | grep -c "${MOTION_SQL_TABLE}$"`

                    if [ ${table_exists} -eq 0 -o ${force} -eq 1 ]
                    then
                        echo

                        get_sql_root_password

                        {
                            echo "USE ${MOTION_SQL_DATABASE};"

                            if [ ${force} -eq 1 ]
                            then
                                echo "DROP TABLE ${MOTION_SQL_TABLE};"
                            fi

                            echo "USE ${MOTION_SQL_DATABASE};"
                            echo "CREATE TABLE IF NOT EXISTS \`${MOTION_SQL_TABLE}\` ("
                            echo "    \`camera\` int(11) default NULL,"
                            echo "    \`filename\` varchar(80) NOT NULL default '',"
                            echo "    \`frame\` int(11) default NULL,"
                            echo "    \`file_type\` int(11) default NULL,"
                            echo "    \`time_stamp\` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,"
                            echo "    \`text_event\` varchar(40) NOT NULL default '0000-00-00 00:00:00',"
                            echo "    \`event_time_stamp\` timestamp NOT NULL default '0000-00-00 00:00:00',"
                            echo "    \`file_size\` varchar(36) NOT NULL default '0',"
                            echo "    PRIMARY KEY \`filename\` (\`filename\`),"
                            echo "    KEY \`time_stamp\` (\`time_stamp\`),"
                            echo "    KEY \`event_time_stamp\` (\`event_time_stamp\`)"
                            echo ") ENGINE=MyISAM DEFAULT CHARSET=latin1;"
                        } > ${sql_cmd_file}
                    else
                        mecho --info "found."
                    fi
                    ;;

                5)
                    step_name="checking if primary key exists"
                    mecho -n "${step_name} ..."
                    key_exists=`/usr/local/mysql/bin/mysql -h${MOTION_SQL_HOST} -u${MOTION_SQL_USER} -p${MOTION_SQL_PASS} -D${MOTION_SQL_DATABASE} -e"SHOW INDEX FROM security;" | grep -c "filename"`

                    if [ ${key_exists} -eq 0 ]
                    then
                        echo

                        get_sql_root_password

                        {
                            echo "USE ${MOTION_SQL_DATABASE};"
                            echo "ALTER TABLE ${MOTION_SQL_TABLE} ADD PRIMARY KEY \`filename\` (\`file name\`);"
                        } > ${sql_cmd_file}
                    else
                        mecho --info "found."
                    fi
                    ;;
            esac

            if [ -f ${sql_cmd_file} ]
            then
                /usr/local/mysql/bin/mysql -h${MOTION_SQL_HOST} -uroot -p${root_pass} < ${sql_cmd_file} 2>${motionconfig_path}/motion-sql-db-results.txt

                if [ $? -eq 0 ]
                then
                    # database created
                    rm -f ${sql_cmd_file}
                    rm -f ${motionconfig_path}/motion-sql-db-results.txt

                    mecho --info "done."
                else
                    # error
                    rm -f ${sql_cmd_file}

                    mecho --error "failed."
                    mecho --warn  "an error appeared while ${step_name}!"
                    break
                fi
            fi
        done
    fi
}

#----------------------------------------------------------------------------------------
# remove SQL table and database
#
# $1 - force or ''
#----------------------------------------------------------------------------------------
remove_sql_db_and_table()
{
    if [ "${MOTION_SQL_ENABLED}" = "yes" ]
    then
        # create sql command file
        root_pass=''
        sql_cmd_file=/var/run/motion-cmd-$$
        for step in 1 2 3
        do
            # delete sql command file
            rm -f ${sql_cmd_file}

            case ${step} in
                1)
                    step_name="removing sql database table"
                    mecho -n "${step_name} ..."
                    table_exists=`/usr/local/mysql/bin/mysql -h${MOTION_SQL_HOST} -u${MOTION_SQL_USER} -p${MOTION_SQL_PASS} -D${MOTION_SQL_DATABASE} -e"SHOW TABLES" | grep -c "${MOTION_SQL_TABLE}$"`

                    if [ ${table_exists} -eq 0 ]
                    then
                        echo
                        get_sql_root_password

                        {
                            echo "USE ${MOTION_SQL_DATABASE};"
                            echo "DROP TABLE ${MOTION_SQL_TABLE};"
                        } > ${sql_cmd_file}
                    else
                        mecho --info "found."
                    fi
                    ;;

                2)
                    step_name="removing sql database"
                    mecho -n "${step_name} ..."
                    db_exists=`/usr/local/mysql/bin/mysql -h${MOTION_SQL_HOST} -u${MOTION_SQL_USER} -p${MOTION_SQL_PASS} -e"SHOW DATABASES" | grep -c "${MOTION_SQL_DATABASE}$"`
                    if [ ${db_exists} -eq 0 ]
                    then
                        echo
                        get_sql_root_password

                        {
                            echo "DROP DATABASE ${MOTION_SQL_DATABASE};"
                        } > ${sql_cmd_file}
                    else
                        mecho --info "found."
                    fi
                    ;;

                3)
                    step_name="removing sql database user"
                    mecho -n "${step_name} ..."
                    /usr/local/mysql/bin/mysql -h${MOTION_SQL_HOST} -u${MOTION_SQL_USER} -p${MOTION_SQL_PASS} -e"QUIT" > /dev/null

                    if [ $? -eq 0 ]
                    then
                        # database can be accessed, go on...
                        echo
                        get_sql_root_password

                        mecho "${step_name}..."

                        {
                            echo "DROP USER '${MOTION_SQL_USER}'@'${MOTION_SQL_HOST}';"
                        } > ${sql_cmd_file}
                    else
                        mecho --info "granted."
                    fi
                    ;;
            esac

            if [ -f ${sql_cmd_file} ]
            then
                /usr/local/mysql/bin/mysql -h${MOTION_SQL_HOST} -uroot -p${root_pass} < ${sql_cmd_file} 2>${motionconfig_path}/motion-sql-db-results.txt

                if [ $? -eq 0 ]
                then
                    # database created
                    rm -f ${sql_cmd_file}
                    rm -f ${motionconfig_path}/motion-sql-db-results.txt

                    mecho --info "done."
                else
                    # error
                    rm -f ${sql_cmd_file}

                    mecho --error "failed."
                    mecho --warn  "an error appeared while ${step_name}!"
                    break
                fi
            fi
        done
    fi
}

#----------------------------------------------------------------------------------------
# extract web files to target_dir
#
# $1 - 'no', 'motionbrowser' or 'motioncgi'
# $2 - 'force'
#----------------------------------------------------------------------------------------
extract_webfiles ()
{
    if [ "${MOTION_WEB_TARGET_PATH}" != "" ]
    then
        mecho "checking web files in '${MOTION_WEB_TARGET_PATH}' ..."

        _ew_force=0
        _ew_version=''
        _ew_type="$1"

        if [ "$2" = "force" ]
        then
            if [ -f ${MOTION_WEB_TARGET_PATH}/motion.ver ]
            then
                if /var/install/bin/ask tty _WARN "web files already exist, overwrite" "n"
                then
                    _ew_force=1
                fi
            else
                _ew_force=1
            fi
        else
            if [ -f ${MOTION_WEB_TARGET_PATH}/motion.ver ]
            then
                # file exists, read version number
                _ew_version=`cat ${MOTION_WEB_TARGET_PATH}/motion.ver`

                if [ "${_ew_version}" = "${motion_version}" ]
                then
                    mecho "- web files are up-to-date."
                else
                    # different version numbers found
                    if /var/install/bin/ask tty _WARN "web files differ, update" "n"
                    then
                        _ew_force=1
                    else    
                        mecho -warn "You won't be asked to update the Motion web files again! Please use"
                        mecho -warn "the 'Force motion web file update' menue entry to force an update!"
                        anykey
                        echo "${motion_version}" > ${MOTION_WEB_TARGET_PATH}/motion.ver
                    fi
                fi
            else
                # file doesn't exist, force update ...
                _ew_force=1
            fi
        fi

        if [ ${_ew_force} -eq 1 ]
        then
            # extract web files
            mecho "- updating web files ..."

            if [ ! -d ${MOTION_WEB_TARGET_PATH} ]
            then
                mkdir -p ${MOTION_WEB_TARGET_PATH}
            fi

            case ${_ew_type} in
                'motionbrowser')
                    tar xzf ${motionconfig_path}/motionbrowser.tgz -C ${MOTION_WEB_TARGET_PATH}
                    ;;

                'motioncgi')
                    tar xzf ${motionconfig_path}/motion-cgi.tgz -C ${MOTION_WEB_TARGET_PATH}
                    cp ${motionconfig_path}/cambozola.jar ${MOTION_WEB_TARGET_PATH}/
                    ;;
            esac

            # set update marker
            echo "${motion_version}" > ${MOTION_WEB_TARGET_PATH}/motion.ver

            # change ownership
            mecho "- setting access rights ..."
            chown -R ${motion_apache_user}  ${MOTION_WEB_TARGET_PATH}
            chgrp -R ${motion_apache_group} ${MOTION_WEB_TARGET_PATH}
        fi
#   else
#       # web-gui has been disabled
    fi
}

#----------------------------------------------------------------------------------------
# remove webfiles from directory
#
# $1 - 'motionbrowser', 'motioncgi' or 'both'
#----------------------------------------------------------------------------------------
remove_webfiles ()
{
    _rw_type=$1

    case ${_rw_type} in
        'motionbrowser')
            _rw_list=${motionconfig_path}/motionbrowser.tgz.lst
            _rw_path=${APACHE2_DOCUMENT_ROOT}/motionbrowser

            if [ -d ${_rw_path} ]
            then
                mecho "looking for obsolete '${_rw_type}' web files ..."
            fi
            ;;
        'motioncgi')
            _rw_list=${motionconfig_path}/motion-cgi.tgz.lst
            _rw_path=${APACHE2_DOCUMENT_ROOT}/motion-cgi

            if [ -d ${_rw_path} ]
            then
                mecho "looking for obsolete '${_rw_type}' web files ..."
            fi
            ;;
        'both')
            _rw_list=''
            remove_webfiles 'motionbrowser'
            remove_webfiles 'motioncgi'
            ;;
    esac

    if [ "${_rw_list}" != ""  -a -f ${_rw_list} ]
    then
        if [ -d ${_rw_path} ]
        then
            # remove directory content
            for FNAME in `cat ${_rw_list}`
            do
                if [ -f ${_rw_path}/${FNAME} ]
                then
                  # echo "remove file: ${_rw_path}/${FNAME}"
                    rm -f ${_rw_path}/${FNAME}
                elif [ -d ${_rw_path}/${FNAME} ]
                then
                  # echo "remove dir: ${_rw_path}/${FNAME}"
                    rmdir --ignore-fail-on-non-empty ${_rw_path}/${FNAME}
                fi
            done

            # remove directory itself
            rmdir --ignore-fail-on-non-empty ${_rw_path}
        fi
    fi
}

#----------------------------------------------------------------------------------------
# check if video device exists - and create it if not
# input: $1 - video device
#----------------------------------------------------------------------------------------
check_videodevice()
{
    vdev="$1"

  # mecho "checking video device (${vdev}) ..."

    echo "${vdev}" | grep -q "^/dev/video"

    if [ $? -eq 0 ]
    then
        # valid naming, go on...
        if [ "${vdev}" = "/dev/video" ]
         then
             vdev='/dev/video0'
             vdevnbr=0
         else
             vdevnbr=`echo "${vdev}" | sed 's/.*video//'`
         fi

         # check if character device exists
         if [ ! -c ${vdev} ]
         then
             mecho "- creating /dev/video${vdevnbr} ..."
             mknod -m 0666 ${vdev} c 81 ${vdevnbr}
         fi

         # check if symbolic link exists
         if [ ! -L /dev/video ]
         then
             ln -s /dev/video0 /dev/video
         fi
  # else
  #     # naming not ok.
  #     mecho -error "video device naming invalid!"
    fi
}

#----------------------------------------------------------------------------------------
# check if mysql has been enabled
# input:  $1 - '-quiet' means no output
# return:  0 - not installed and activated
#          1 - installed and activated
#----------------------------------------------------------------------------------------
check_installed_mysql ()
{
    retval=0
    if [ "${MOTION_SQL_ENABLED}" = "yes" -a -f ${mysqlfile} ]
    then
        # mysql installed
        . ${mysqlfile}

        if [ "${START_MYSQL}" = "yes" ]
        then
            # mysql activated
            if [ "$1" != "-quiet" ]
            then
                mecho "mysql support has been enabled ..."
            fi
            retval=1
        else
            # mysql deactivated
            if [ "$1" != "-quiet" ]
            then
                mecho -warn "mysql support has been disabled ..."
            fi
        fi
    fi

    return ${retval}
}

#----------------------------------------------------------------------------------------
# check if the sql database backup has been activated
# return:  0 - not installed and activated
#          1 - installed and activated
#----------------------------------------------------------------------------------------
check_sql_backup()
{
    _csb_retval=1

    if [ ${MYSQL_INSTALLED} -eq 1 ]
    then
        . ${mysqlfile}

        _csb_idx=1
        while [ ${_csb_idx} -le ${MYSQL_BACKUP_N} ]
        do
            eval _csb_file='$MYSQL_BACKUP_'${_csb_idx}'_FILE'

            if [ "${_csb_file}" = "${MOTION_SQL_DATABASE}" ]
            then
                _csb_retval=0
                break
            fi

            _csb_idx=`expr ${_csb_idx} + 1`
        done

        if [ ${_csb_retval} -ne 0 ]
        then
            _csb_count=`expr ${MYSQL_BACKUP_N} + 1`

            write_to_config_log -warn "The SQL database used by Motion will currently not be backuped!"
            write_to_config_log -warn -ff "It's recommended to set the following mysql parameters:"
            write_to_config_log -warn -ff "MYSQL_BACKUP_N='${_csb_count}'"
            write_to_config_log -warn -ff " MYSQL_BACKUP_${_csb_count}_FILE='motiondb'"
            write_to_config_log -warn -ff " MYSQL_BACKUP_${_csb_count}_MAX='7'"
            write_to_config_log -warn -ff " MYSQL_BACKUP_${_csb_count}_FORCE='no"
        fi
    fi

    return ${_csb_retval}
}

#----------------------------------------------------------------------------------------
# creating motionbrowser configuration
#----------------------------------------------------------------------------------------
create_motionbrowser_conf ()
{
    if [ "${START_MOTION}" = "yes" ]
    then
        # default parameters
        generate_motionbrowserconf="${MOTION_WEB_TARGET_PATH}/config.inc"

        mecho "creating motionbrowser configuration file ..."

        {
            echo '<?php'
            echo '/* ---------------------------------------------------------------------'
            echo '    config.inc - MotionBrowser - User interface for the result of'
            echo '                 Motion application. Developed by Carlos Ladeira'
            echo '                 (caladeira@gmail.com)'
            echo
            echo '    Creation:     2006-03-11  Carlos Ladeira'
            echo "    Last Update:  ${EISDATE} ${EISTIME} jed"
            echo
            echo '    For more details, please visit:'
            echo '    http://www.lavrsen.dk/twiki/bin/view/Motion/WebHome'
            echo
            echo '    This program is free software; you can redistribute it and/or modify'
            echo '    it under the terms of the GNU General Public License as published by'
            echo '    the Free Software Foundation; either version 2 of the License, or'
            echo '    (at your option) any later version..'
            echo '   -------------------------------------------------------------------- */'
            echo
            echo '    // general constants e configuration values'
            echo '    $lang="de";'
            echo '    $locales="de_DE@euro";'
            echo
            echo '    // mysql related data'
            echo "    \$sql_db   = \"${MOTION_SQL_DATABASE}\";"
            echo "    \$sql_host = \"${MOTION_SQL_HOST}\";"
            echo "    \$sql_user = \"${MOTION_SQL_USER}\";"
            echo "    \$sql_pass = \"${MOTION_SQL_PASS}\";"
            echo
            echo '    // info for the webcam popups'
            echo "    \$server_name = \$_SERVER['SERVER_NAME'];"
            echo "    \$webcam_x = 640;"
            echo "    \$webcam_y = 480;"

            idx=1
            wc_portlist=''
            while [ ${idx} -le ${MOTION_N} ]
            do
                eval active='$MOTION_'${idx}'_ACTIVE'

                if [ "${active}" = "yes" ]
                then
                    eval wc_port='$MOTION_'${idx}'_WEBCAM_PORT'

                    if [ "${wc_portlist}" = "" ]
                    then
                        wc_portlist="${wc_port}"
                    else
                        wc_portlist="${wc_portlist}, ${wc_port}"
                    fi
                fi

                idx=`expr ${idx} + 1`
            done

            echo "    \$webcam_port = array(${wc_portlist});"
            echo
            echo '    // setup port'
            echo "    \$setup_port = ${MOTION_CONTROL_PORT};"
            echo
            echo '    // default size for the thumbnails'
            echo '    $thumb_width  = 88;'
            echo '    $thumb_height = 72;'
            echo
            echo '    // default size for the disc space bar'
            echo '    $quota_width  = 800;'
            echo '    $quota_height = 20;'
            echo
            echo '    // surveillance popup window'
            echo '    $cam_windows_x = 3;'
            echo '    $cam_windows_y = 1;'
            echo '?>'
        } > ${generate_motionbrowserconf}
    fi
}

#----------------------------------------------------------------------------------------
# creating motion-cgi configuration
#----------------------------------------------------------------------------------------
create_motion_cgi_conf ()
{
    if [ "${START_MOTION}" = "yes" ]
    then
        # default parameters
        mecho "creating motion-cgi configuration file ..."

        {
            #----------------------------------------------------------------------------------------
            print_short_header "${generate_motioncgiconf}" "${pgmname}" "${motion_version}"
            #----------------------------------------------------------------------------------------

            # absolute path of the motion-cgi files
            # ABSOLUTEPATH=/var/www/htdocs/motion-cgi
            echo "ABSOLUTEPATH=${APACHE2_DOCUMENT_ROOT}/motion-cgi"
            echo "CAMERAPATH=../motion"
            echo "IMAGEDIR=/motion-cgi/images"

            # absolute path to the motion binary file
            echo "MOTIONBIN=/usr/local/bin"
            echo "CONTROLPORT=${MOTION_CONTROL_PORT}"

            sidx=1
            while [ ${sidx} -le ${MOTION_N} ]
            do
                eval active='$MOTION_'${sidx}'_ACTIVE'

                if [ "${active}" = "yes" ]
                then
                    eval wport='$MOTION_'${sidx}'_WEBCAM_PORT'
                    eval ltext='$MOTION_'${sidx}'_TEXT_LEFT'

                    if [ "${ltext}" = "" ]
                    then
                        ltext="camera-${sidx}"
                    fi

                    echo "SOURCE=camera-${sidx}=webcam-live.cgi?${wport}=${sidx}=${ltext}"
                fi

                sidx=`expr ${sidx} + 1`
            done
        } > ${generate_motioncgiconf}
    fi
}

#----------------------------------------------------------------------------------------
# creating motion wgetrc-configuration
#----------------------------------------------------------------------------------------
create_motion_wgetrc_conf ()
{
    if [ "${START_MOTION}" = "yes" ]
    then
        # default parameters
        mecho "creating motion wgetrc-configuration file ..."

        {
            #----------------------------------------------------------------------------------------
            print_short_header "${generate_motionwgetrc}" "${pgmname}" "${motion_version}"
            #----------------------------------------------------------------------------------------
            echo "http_user=${MOTION_CONTROL_USER}"
            echo "http_password=${MOTION_CONTROL_PASS}"
        } > ${generate_motionwgetrc}

        chmod 0400 ${generate_motionwgetrc}
        chown ${motion_apache_user}  ${generate_motionwgetrc}
        chgrp ${motion_apache_group} ${generate_motionwgetrc}
    fi
}

#----------------------------------------------------------------------------------------
# check for activated USB package
#----------------------------------------------------------------------------------------
check_active_usb ()
{
    ret=1
    if [ -f ${usbfile} ]
    then
        # config file exists
        . ${usbfile}

        if [ "${START_USB}" = "yes" ]
        then
            # USB configuration has been activated
            ret=0
        fi
    fi

    return ${ret}
}

#----------------------------------------------------------------------------------------
# check for any active configuration
#----------------------------------------------------------------------------------------
check_active_config ()
{
    ret=1

    if [ ${MOTION_N} -ge 1 ]
    then
        cidx=1
        while [ ${cidx} -le ${MOTION_N} ]
        do
            eval active='$MOTION_'${cidx}'_ACTIVE'

            if [ "${active}" = "yes" ]
            then
                # active configuration found
                ret=0
                break
            fi

            cidx=`expr ${cidx} + 1`
        done
    fi

    return ${ret}
}

#----------------------------------------------------------------------------------------
# check if tcp port is unique
# input : $1 - nbr of config set
# return: usable uniq port
#----------------------------------------------------------------------------------------
get_uniq_webcamport ()
{
    cidx=$1
    eval curr_port='$MOTION_'${cidx}'_WEBCAM_PORT'

    if [ "${curr_port}" = "" ]
    then
        # use default value
        curr_port=${MOTION_DEF_WEBCAM_PORT}
    fi

    pidx=1
    quit_flag=1
    while [ ${quit_flag} -ne 0 ]
    do
        while [ ${pidx} -le ${MOTION_N} ]
        do
            if [ ${pidx} -ne ${cidx} ]
            then
                # not equal
                eval active='$MOTION_'${pidx}'_ACTIVE'

                if [ "${active}" = "yes" ]
                then
                    eval port='$MOTION_'${pidx}'_WEBCAM_PORT'

                    if [ "${curr_port}" = "${port}" ]
                    then
                        curr_port=`expr ${curr_port} + 1`
                        pidx=0
                    fi
                fi
            fi

            pidx=`expr ${pidx} + 1`
        done

        if [ ${pidx} -ge ${MOTION_N} ]
        then
            quit_flag=0
        fi
    done

    echo "${curr_port}"
}

#----------------------------------------------------------------------------------------
# prefix script names with executation path
# input:  $1 - execution string
# return: modified execution string
#----------------------------------------------------------------------------------------
add_script_path ()
{
    echo "$1" | sed -e "s#^[ ;]*#${motionconfig_path}/#" -e 's/[ ;]*$//' -e "s# *; *#;${motionconfig_path}/#g"
}

#----------------------------------------------------------------------------------------
# check if script exists and is executable
# input:  $1 - execution string
# return:  0 - all scripts exist and are executable
#          1 - one or more scripts don't exist or are not executable
#----------------------------------------------------------------------------------------
check_script_exists_and_executable ()
{
    _csee_str="$1"

    _csee_ret=0
    _csee_pos=1
    while [ ${_csee_pos} -gt 0 ]
    do
        _csee_pos=`expr index "${_csee_str}" ";"`

        if [ ${_csee_pos} -gt 0 ]
        then
            _csee_len=`expr ${_csee_pos} - 1`
            _csee_script="`expr substr "${_csee_str}" 1 ${_csee_len} | cut -d' ' -f1`"
            _csee_str=`echo "${_csee_str}" | cut -d';' -f2-`
        else
            _csee_script="`echo "${_csee_str}" | cut -d' ' -f1`"
        fi

        if [ ! -x ${_csee_script} ]
        then
            _csee_ret=1
        fi
    done

    return ${_csee_ret}
}

#----------------------------------------------------------------------------------------
# creating motion configuration
#----------------------------------------------------------------------------------------
create_motion_config ()
{
    if [ "${START_MOTION}" = "yes" ]
    then
        # default parameters
        mecho "creating motion configuration file ..."

        # check pre-/post-start scripts
        if [ "${MOTION_COMMAND_PRESTART}" != "" ]
        then
            if [ ! -x ${MOTION_COMMAND_PRESTART} ]
            then
                # file doesn't exist
                write_to_config_log -error "Parameter MOTION_COMMAND_PRESTART='${MOTION_COMMAND_PRESTART}'"
                write_to_config_log -error -ff "has been set but the file doesn't exist or has no execute rights!"
            fi
        fi

        if [ "${MOTION_COMMAND_POSTSTART}" != "" ]
        then
            if [ ! -x ${MOTION_COMMAND_POSTSTART} ]
            then
                # file doesn't exist
                write_to_config_log -error "Parameter MOTION_COMMAND_POSTSTART='${MOTION_COMMAND_POSTSTART}'"
                write_to_config_log -error -ff "has been set but the file doesn't exist or has no execute rights!"
            fi
        fi

        {
            #----------------------------------------------------------------------------------------
            print_short_header "${generate_motionconf}" "${pgmname}" "${motion_version}"
            #----------------------------------------------------------------------------------------

            # !! The following parameters are not allowed to be used in a thread specific
            # !! configuration file and must therefore be placed in motion.conf:
            # !!
            # !! control_authentication, control_html_output, control_localhost,
            # !! control_port, daemon, and thread.

            echo '#------------------------------------------------------------------'
            echo '# Daemon'
            echo '#------------------------------------------------------------------'

            # Start in daemon (background) mode and release terminal (default: off)

            if [ "${MOTION_DO_DEBUG}" = "yes" ]
            then
                # debug mode
                echo 'daemon off'
            else
                echo 'daemon on'
            fi

            # File to store the process ID, also called pid file. (default: not defined)

            echo "process_id_file ${pidfile}"

            #----------------------------------------------------------------------------------------
            # Basic setup mode
            #----------------------------------------------------------------------------------------

            # Start in Setup-Mode, daemon disabled. (default: off)
            if [ "${MOTION_SETUPMODE_ENABLED}" = "yes" ]
            then
                echo 'setup_mode on'
            else
                echo 'setup_mode off'
            fi

            echo
            echo '#------------------------------------------------------------------'
            echo '# Capture device options'
            echo '#------------------------------------------------------------------'

            # Videodevice to be used for capturing  (default /dev/video0)
            # for FreeBSD default is /dev/bktr0
            #
            # videodevice /dev/video0
            #
            # WILL BE SET IN THREAD CONFIGURATION

          # echo "videodevice /dev/video0"

            # v4l2_palette allow to choose preferable palette to be use by motion
            # to capture from those supported by your videodevice. ( default: 8)
            # i.ex if your videodevice supports V4L2_PIX_FMT_SBGGR8 and
            # V4L2_PIX_FMT_MJPEG by default motion will use V4L2_PIX_FMT_MJPEG so
            # set v4l2_palette 1 to force motion use V4L2_PIX_FMT_SBGGR8 instead.
            #
            # Values :
            #
            # V4L2_PIX_FMT_SN9C10X : 0  'S910'
            # V4L2_PIX_FMT_SBGGR8  : 1  'BA81'
            # V4L2_PIX_FMT_MJPEG   : 2  'MJPEG'
            # V4L2_PIX_FMT_JPEG    : 3  'JPEG'
            # V4L2_PIX_FMT_RGB24   : 4  'RGB3'
            # V4L2_PIX_FMT_UYVY    : 5  'UYVY'
            # V4L2_PIX_FMT_YUYV    : 6  'YUYV'
            # V4L2_PIX_FMT_YUV422P : 7  '422P'
            # V4L2_PIX_FMT_YUV420  : 8  'YU12'
            #
            # v4l2_palette 8

          # echo "v4l2_palette 8"

            # Tuner device to be used for capturing using tuner as source (default /dev/tuner0)
            # This is ONLY used for FreeBSD. Leave it commented out for Linux
            #
            # ; tunerdevice /dev/tuner0
        
          # echo '; tunerdevice /dev/tuner0'

            # The video input to be used (default: 8)
            # Many TV tuner cards have the input channels: TV Tuner = 0, Standard composite
            # video = 1, S-VHS = 3. Other have TV=0, composite video 1= 1, composite video = 2,
            # S-VHS = 3. It is recommended to set the parameter to 8 for USB cameras as your
            # first try. For video/TV capture cards input 1 is normally the composite video
            # input.
            #
            # input 8
            #
            # WILL BE SET IN THREAD CONFIGURATION

          # echo "input 8"

            # The video norm to use (only for video capture and TV tuner cards)
            # Values: 0 (PAL), 1 (NTSC), 2 (SECAM), 3 (PAL NC no colour). Default: 0 (PAL)
            #
            # 'norm 0'

            echo "norm 0"

            # The frequency to set the tuner to (kHz) (only for TV tuner cards) (default: 0)
            #
            # frequency 0

            echo "frequency ${MOTION_DEF_VIDEO_FREQ}"

            # Rotate image this number of degrees. The rotation affects all saved images as
            # well as mpeg movies. Valid values: 0 (default = no rotation), 90, 180 and 270.
            #
            # rotate 0

            echo "rotate ${MOTION_DEF_VIDEO_ROTATE}"

            # Image width (pixels). Valid range: Camera dependent, default: 352
            #
            # width 320

            echo "width ${MOTION_DEF_VIDEO_WIDTH}"

            if [ `expr ${MOTION_DEF_VIDEO_WIDTH} % 16` -ne 0 ]
            then
                write_to_config_log -error "Parameter MOTION_DEF_VIDEO_WIDTH='${MOTION_DEF_VIDEO_WIDTH}' has been set"
                write_to_config_log -error -ff "to a non modulo 16 value."
            fi

            # Image height (pixels). Valid range: Camera dependent, default: 288
            #
            # height 240

            echo "height ${MOTION_DEF_VIDEO_HEIGHT}"

            if [ `expr ${MOTION_DEF_VIDEO_HEIGHT} % 16` -ne 0 ]
            then
                write_to_config_log -error "Parameter MOTION_DEF_VIDEO_HEIGHT='${MOTION_DEF_VIDEO_HEIGHT}' has been set"
                write_to_config_log -error -ff "to a non modulo 16 value."
            fi

            # Maximum number of frames to be captured per second.
            # Valid range: 2-100. Default: 100 (almost no limit).
            #
            # framerate 2

            echo "framerate ${MOTION_DEF_VIDEO_FRAMERATE}"

            # Minimum time in seconds between capturing picture frames from the camera.
            # Default: 0 = disabled - the capture rate is given by the camera framerate.
            # This option is used when you want to capture images at a rate lower than
            # 2 per second.
            #
            # minimum_frame_time 0

            echo "minimum_frame_time ${MOTION_DEF_VIDEO_FRAMETIME}"

            # URL to use if you are using a network camera, size will be autodetected
            # (incl http:// ftp:// or file:///) Must be a URL that returns single jpeg
            # pictures or a raw mjpeg stream. Default: Not defined
            #
            # Axis webcam (only two valid picture sizes are available: 320x240 and 640x480
            #   To increase the performance use this urls:
            #
            #   for multipart jpeg
            #     http://192.168.1.10/axis-cgi/mjpg/video.cgi?showlength=1
            #   or for single jpeg
            #     http://192.168.1.10/axis-cgi/jpg/image.cgi?showlength=1
            #
            # ; netcam_url value
            #
            # WILL BE SET IN THREAD CONFIGURATION

          # echo '; netcam_url value'

            # Username and password for network camera (only if required).
            # Default: not defined
            # Syntax is user:password
            #
            # ; netcam_userpass value
            #
            # WILL BE SET IN THREAD CONFIGURATION

          # echo '; netcam_userpass value'


            # The setting for keep-alive of network socket, should improve performance
            # on compatible net cameras.
            # 1.0        : the historical implementation using HTTP/1.0, closing the
            #              socket after each http request.
            # 1.1        : Use HTTP/1.1 requests that support keep alive as default.
            # keep_alive : Use HTTP/1.0 requests with keep alive header to reuse the
            #              same connection.
            # Default: 1.0
            #
            # ; netcam_http value
            #
            # URL to use for a netcam proxy server, if required, e.g. "http://myproxy".
            # If a port number other than 80 is needed, use "http://myproxy:1234".
            # Default: not defined

            if [ "${HTTP_PROXY}" != "" ]
            then
                echo 'netcam_http  1.1'
                echo "netcam_proxy \"${HTTP_PROXY}\""
            fi

            # Let motion regulate the brightness of a video device (default: off).
            # The auto_brightness feature uses the brightness option as its target value.
            # If brightness is zero auto_brightness will adjust to average brightness
            # value 128. Only recommended for cameras without auto brightness
            #
            # auto_brightness off
            #
            # Set the initial brightness of a video device.
            # If auto_brightness is enabled, this value defines the average brightness level
            # which Motion will try and adjust to.
            # Valid range 0-255, default 0 = disabled
            #
            # brightness 0

            echo "${MOTION_DEF_VIDEO_BRIGHTNESS}" | grep -q "auto:"

            if [ $? -eq 0 ]
            then
                # multiple parameters given
                echo 'auto_brightness on'
                echo "brightness `echo ${MOTION_DEF_VIDEO_BRIGHTNESS} | cut -d: -f2`"
            else
                echo 'auto_brightness off'
                echo "brightness ${MOTION_DEF_VIDEO_BRIGHTNESS}"
            fi

            # Set the contrast of a video device.
            # Valid range 0-255, default 0 = disabled
            #
            # contrast 0

            echo "contrast ${MOTION_DEF_VIDEO_CONTRAST}"

            # Set the saturation of a video device.
            # Valid range 0-255, default 0 = disabled
            #
            # saturation 0

            echo "saturation ${MOTION_DEF_VIDEO_SATURATION}"

            # Set the hue of a video device (NTSC feature).
            # Valid range 0-255, default 0 = disabled

            echo 'hue 0'

            echo
            echo '#------------------------------------------------------------------'
            echo '# Round robin (multiple inputs on same video device name)'
            echo '#------------------------------------------------------------------'

            # Number of frames to capture in each roundrobin step (default: 1)

            echo 'roundrobin_frames 1'

            # Number of frames to skip before each roundrobin step (default: 1)

            echo 'roundrobin_skip 1'

            # Try to filter out noise generated by roundrobin (default: off)

            echo 'switchfilter off'

            echo
            echo '#------------------------------------------------------------------'
            echo '# Motion detection settings'
            echo '#------------------------------------------------------------------'

            # Threshold for number of changed pixels in an image that
            # triggers motion detection (default: 1500)
            #
            # threshold 1500

            echo "threshold ${MOTION_DEF_MOTION_TRIGGER}"

            # Automatically tune the threshold down if possible (default: off)
            #
            # threshold_tune off
            #
            #  Better don't enable this option it's broken -- AngelCarpintero - 07 May 2008 

          # if [ "${MOTION_DEF_MOTION_TRIGGER_TUNE}" = "yes" ]
          # then
          #     echo 'threshold_tune on'
          # else
                echo 'threshold_tune off'
          # fi

            # Automatically tune the noise threshold (default: on)
            #
            # noise_tune on
            #
            # Noise threshold for the motion detection (default: 32)
            #
            # noise_level 32

            if [ "${MOTION_DEF_MOTION_NOISE_TUNE}" = "no" ]
            then
                echo 'noise_tune off'

                echo "noise_level ${MOTION_DEF_MOTION_NOISE_LEVEL}"
            else
                # parameter has been set to 'yes' therefore the noise_level
                # value is ignored
                echo 'noise_tune on'
            fi

            # Despeckle motion image using (e)rode or (d)ilate or (l)abel (Default: not defined)
            # Recommended value is EedDl. Any combination (and number of) of E, e, d, and D is
            # valid. (l)abeling must only be used once and the 'l' must be the last letter.
            # Comment out to disable

            echo 'despeckle EedDl'

            # Detect motion in predefined areas (1 - 9). Areas are numbered like that:  1 2 3
            # A script (on_area_detected) is started immediately when motion is         4 5 6
            # detected in one of the given areas, but only once during an event.        7 8 9
            # One or more areas can be specified with this option. (Default: not defined)
            #
            # ; area_detect value

            # PGM file to use as a sensitivity mask.
            # Full path name to. (Default: not defined)
            #
            # ; mask_file value

            if [ "${MOTION_DEF_MASK_FILE}" != "" ]
            then
                if [ -f ${motionconfig_path}/${MOTION_DEF_MASK_FILE} ]
                then
                    echo "mask_file ${motionconfig_path}/${MOTION_DEF_MASK_FILE}"
                else
                    # file doesn't exist
                    write_to_config_log -warn "Parameter MOTION_DEF_MASK_FILE='${MOTION_DEF_MASK_FILE}'"
                    write_to_config_log -warn -ff "has been set but the file doesn't exist!"
                fi
            fi

            # Dynamically create a mask file during operation (default: 0)
            # Adjust speed of mask changes from 0 (off) to 10 (fast)
            #
            # smart_mask_speed 0

            echo "smart_mask_speed ${MOTION_DEF_MASK_SPEED}"

            # Ignore sudden massive light intensity changes given as a percentage of the picture
            # area that changed intensity. Valid range: 0 - 100 , default: 0 = disabled
            #
            # lightswitch 0

            echo "lightswitch ${MOTION_DEF_MOTION_LIGHTSWITCH}"

            # Picture frames must contain motion at least the specified number of frames
            # in a row before they are detected as true motion. At the default of 1, all
            # motion is detected. Valid range: 1 to thousands, recommended 1-10
            #
            # minimum_motion_frames 1

            echo "minimum_motion_frames ${MOTION_DEF_MOTION_FRAMES}"

            # Specifies the number of pre-captured (buffered) pictures from before motion
            # was detected that will be output at motion detection.
            # Recommended range: 0 to 5 (default: 0)
            # Do not use large values! Large values will cause Motion to skip video frames and
            # cause unsmooth mpegs. To smooth mpegs use larger values of post_capture instead.
            #
            # pre_capture 0

            echo "pre_capture ${MOTION_DEF_MOTION_PRECAPTURE}"

            # Number of frames to capture after motion is no longer detected (default: 0)
            #
            # post_capture 0

            echo "post_capture ${MOTION_DEF_MOTION_POSTCAPTURE}"

            # Gap is the seconds of no motion detection that triggers the end of an event
            # An event is defined as a series of motion images taken within a short timeframe.
            # Recommended value is 60 seconds (Default). The value 0 is allowed and disables
            # events causing all Motion to be written to one single mpeg file and no pre_capture.
            #
            # gap 60

            echo "gap ${MOTION_DEF_MOTION_ENDGAP}"

            # Maximum length in seconds of an mpeg movie
            # When value is exceeded a new mpeg file is created. (Default: 0 = infinite)
            #
            # max_mpeg_time 0

            echo "max_mpeg_time ${MOTION_DEF_CAPTURE_LENGTH}"

            # Always save images even if there was no motion (default: off)
            #
            # output_all off

            if [ "${MOTION_DEF_CAPTURE_ALL}" = "yes" ]
            then
                echo 'output_all on'
            else
                echo 'output_all off'
            fi

            echo
            echo '#------------------------------------------------------------------'
            echo '# Image file output'
            echo '#------------------------------------------------------------------'

            # Output 'normal' pictures when motion is detected (default: on)
            # Valid values: on, off, first, best
            # When set to 'first', only the first picture of an event is saved.
            # Picture with most motion of an event is saved when set to 'best'.
            # Can be used as preview shot for the corresponding movie.

            case "${MOTION_DEF_CAPTURE_IMAGE}" in
                'best')
                    echo 'output_normal best'
                    ;;
                'first')
                    echo 'output_normal first'
                    ;;
                'no')
                    echo 'output_normal off'
                    ;;
                *)  # yes
                    echo 'output_normal on'
                    ;;
            esac

            # Output pictures with only the pixels moving object (ghost images) (default: off)

            echo 'output_motion off'

            # The quality (in percent) to be used by the jpeg compression (default: 75)

            echo "quality ${MOTION_DEF_CAPTURE_QUALITY}"

            # Output ppm images instead of jpeg (default: off)

            echo 'ppm off'

            echo
            echo '#------------------------------------------------------------------'
            echo '# FFMPEG related options'
            echo '#------------------------------------------------------------------'

            # Film (mpeg) file output, and deinterlacing of the video input'
            # The options movie_filename and timelapse_filename are also used'
            # by the ffmpeg feature'

            # Use ffmpeg to encode mpeg movies in realtime (default: off)

            if [ "${MOTION_DEF_CAPTURE_MOVIE}" = "yes" ]
            then
                echo 'ffmpeg_cap_new on'
            else
                echo 'ffmpeg_cap_new off'
            fi

            # Use ffmpeg to make movies with only the pixels moving
            # object (ghost images) (default: off)

            echo 'ffmpeg_cap_motion off'

            # Use ffmpeg to encode a timelapse movie 
            # Default value 0 = off - else save frame every Nth second
            #
            # The file rollover mode of the timelapse video
            # Valid values: hourly, daily (default), weekly-sunday, weekly-monday, monthly, manual
            #
            # ffmpeg_timelapse 0
            # ffmpeg_timelapse_mode daily

            echo "${MOTION_DEF_CAPTURE_TIMELAPS}" | grep -q ":"

            if [ $? -eq 0 ]
            then
                echo "ffmpeg_timelapse `echo ${MOTION_DEF_CAPTURE_TIMELAPS} | cut -d: -f1`"
                echo "ffmpeg_timelapse_mode `echo ${MOTION_DEF_CAPTURE_TIMELAPS} | cut -d: -f2`"
            else
                echo "ffmpeg_timelapse ${MOTION_DEF_CAPTURE_TIMELAPS}"
                echo 'ffmpeg_timelapse_mode daily'
            fi

            # Bitrate to be used by the ffmpeg encoder (default: 400000)
            # This option is ignored if ffmpeg_variable_bitrate is not 0 (disabled)

            echo 'ffmpeg_bps 500000'

            # Enables and defines variable bitrate for the ffmpeg encoder.
            # ffmpeg_bps is ignored if variable bitrate is enabled.
            # Valid values: 0 (default) = fixed bitrate defined by ffmpeg_bps,
            # or the range 2 - 31 where 2 means best quality and 31 is worst.

            echo 'ffmpeg_variable_bitrate 0'

            # Codec to used by ffmpeg for the video compression.
            # Timelapse mpegs are always made in mpeg1 format independent from this option.
            # Supported formats are: mpeg1 (ffmpeg-0.4.8 only), mpeg4 (default), and msmpeg4.
            # mpeg1 - gives you files with extension .mpg
            # mpeg4 or msmpeg4 - give you files with extension .avi
            # msmpeg4 is recommended for use with Windows Media Player because
            # it requires no installation of codec on the Windows client.
            # swf - gives you a flash film with extension .swf

            echo "ffmpeg_video_codec ${MOTION_DEF_CAPTURE_CODEC}"

            # Use ffmpeg to deinterlace video. Necessary if you use an analog camera
            # and see horizontal combing on moving objects in video or pictures.
            # (default: off)

            echo 'ffmpeg_deinterlace off'

            echo
            echo '#------------------------------------------------------------------'
            echo '# Snapshots (traditional periodic webcam file output)'
            echo '#------------------------------------------------------------------'

            # Make automated snapshot every N seconds (default: 0 = disabled)

            echo "${MOTION_DEF_CAPTURE_SNAPSHOT}" | grep -q "^single:"

            if [ $? -eq 0 ]
            then
                # only one snapshoot will be saved each interval 
                m_def_singlesnap=1
                echo "snapshot_interval `echo ${MOTION_DEF_CAPTURE_SNAPSHOT} | cut -d: -f2`"
            else
                # individual snapshots will be saved each interval
                m_def_singlesnap=0
                echo "snapshot_interval ${MOTION_DEF_CAPTURE_SNAPSHOT}"
            fi

            # Locate and draw a box around the moving object.
            # Valid values: on, off and preview (default: off)
            # Set to 'preview' will only draw a box in preview_shot pictures.
            #
            # locate off

            case "${MOTION_DEF_MOTION_LOCATE}" in
                'yes')
                    echo "locate on"
                    ;;
                'no')
                    echo "locate off"
                    ;;
                'preview')
                    echo "locate preview"
                    ;;
            esac

            echo
            echo '#------------------------------------------------------------------'
            echo '# Text Display'
            echo '#------------------------------------------------------------------'

            # %Y = year, %m = month, %d = date,
            # %H = hour, %M = minute, %S = second, %T = HH:MM:SS,
            # %v = event, %q = frame number, %t = thread (camera) number,
            # %D = changed pixels, %N = noise level, \n = new line,
            # %i and %J = width and height of motion area,
            # %K and %L = X and Y coordinates of motion center
            # %C = value defined by text_event - do not use with text_event!
            # You can put quotation marks around the text to allow
            # leading spaces

            # Draw a user defined text on the images using same options as C function strftime(3)
            # Default: Not defined = no text
            # Text is placed in lower left corner
            #
            # text_left CAMERA %t

            echo "text_left  ${MOTION_DEF_TEXT_LEFT}"

            # Draws the timestamp using same options as C function strftime(3)
            # Default: %Y-%m-%d\n%T = date in ISO format and time in 24 hour clock
            # Text is placed in lower right corner
            #
            # text_right %Y-%m-%d\n%T-%q

            echo "text_right ${MOTION_DEF_TEXT_RIGHT}"

            # Draw characters at twice normal size on images. (default: off)

            if [ "${MOTION_DEF_TEXT_SIZE}" = "double" ]
            then
                echo 'text_double on'
            else
                echo 'text_double off'
            fi

            # Draw the number of changed pixed on the images (default: off)
            # Will normally be set to off except when you setup and adjust the motion settings
            # Text is placed in upper right corner

            echo 'text_changes off'

            # This option defines the value of the special event conversion specifier %C
            # You can use any conversion specifier in this option except %C. Date and time
            # values are from the timestamp of the first image in the current event.
            # Default: %Y%m%d%H%M%S
            # The idea is that %C can be used filenames and text_left/right for creating
            # a unique identifier for each event.

            echo 'text_event %Y%m%d%H%M%S'

            echo
            echo '#------------------------------------------------------------------'
            echo '# Target Directories and filenames For Images And Films'
            echo '#------------------------------------------------------------------'

            # Target base directory for pictures and films
            # Recommended to use absolute path. (Default: current working directory)
            #
            # WILL BE SET IN THREAD CONFIGURATION

            if [ "${MOTION_DEF_TARGET_PATH}" != "" ]
            then
                if [ ! -d "${MOTION_DEF_TARGET_PATH}" ]
                then
                    # create target directory
                    mkdir -p ${MOTION_DEF_TARGET_PATH}

                    # change ownership
                    chown -R ${motion_apache_user}  ${MOTION_DEF_TARGET_PATH}
                    chgrp -R ${motion_apache_group} ${MOTION_DEF_TARGET_PATH}

                    chown -R ${motion_apache_user}  ${MOTION_WEB_TARGET_PATH}
                    chgrp -R ${motion_apache_group} ${MOTION_WEB_TARGET_PATH}

                    # create symbolic link to ease directory access
                    ln -sf ${MOTION_DEF_TARGET_PATH} ${motionconfig_path}/cams
                    ln -sf ${MOTION_WEB_TARGET_PATH} ${motionconfig_path}/web
                fi

              # echo "target_dir ${MOTION_DEF_TARGET_PATH}"
            fi

            # For the options snapshot_, jpeg_, mpeg_ and timelapse_filename'
            # you can use conversion specifiers'
            # %Y = year, %m = month, %d = date,'
            # %H = hour, %M = minute, %S = second,'
            # %v = event, %q = frame number, %t = thread (camera) number,'
            # %D = changed pixels, %N = noise level,'
            # %i and %J = width and height of motion area,'
            # %K and %L = X and Y coordinates of motion center'
            # %C = value defined by text_event'
            # Quotation marks round string are allowed.'

            # File path for snapshots (jpeg or ppm) relative to target_dir
            # Default: %v-%Y%m%d%H%M%S-snapshot
            # Default value is equivalent to legacy oldlayout option
            # For Motion 3.0 compatible mode choose: %Y/%m/%d/%H/%M/%S-snapshot
            # File extension .jpg or .ppm is automatically added so do not include this.
            # Note: A symbolic link called lastsnap.jpg created in the target_dir will always
            # point to the latest snapshot, unless snapshot_filename is exactly 'lastsnap'

          # echo 'snapshot_filename %Y%m%d%H%M%S-%v-snapshot'

            if [ ${m_def_singlesnap} -eq 1 ]
            then
                # only one snapshoot will be saved each interval 
                echo 'snapshot_filename lastsnap'
            else
                # individual snapshoots will be saved each interval 
                echo 'snapshot_filename %Y/%m/%d/%H/%M/%S-snapshot'
            fi

            # File path for motion triggered images (jpeg or ppm) relative to target_dir
            # Default: %v-%Y%m%d%H%M%S-%q
            # Default value is equivalent to legacy oldlayout option
            # For Motion 3.0 compatible mode choose: %Y/%m/%d/%H/%M/%S-%q
            # File extension .jpg or .ppm is automatically added so do not include this
            # Set to 'preview' together with best-preview feature enables special naming
            # convention for preview shots. See motion guide for details

          # echo 'jpeg_filename %v-%Y%m%d%H%M%S-%q'
            echo 'jpeg_filename %Y/%m/%d/%H/%M/%S-%q-%D'

            # File path for motion triggered ffmpeg films (mpeg) relative to target_dir
            # Default: %v-%Y%m%d%H%M%S
            # Default value is equivalent to legacy oldlayout option
            # For Motion 3.0 compatible mode choose: %Y/%m/%d/%H%M%S
            # File extension .mpg or .avi is automatically added so do not include this
            # This option was previously called ffmpeg_filename

          # echo 'movie_filename %Y%m%d%H%M%S-%v'
            echo 'movie_filename %Y/%m/%d/%H%M%S'

            # File path for timelapse mpegs relative to target_dir
            # Default: %Y%m%d-timelapse
            # Default value is near equivalent to legacy oldlayout option
            # For Motion 3.0 compatible mode choose: %Y/%m/%d-timelapse
            # File extension .mpg is automatically added so do not include this

          # echo 'timelapse_filename %Y%m%d-timelapse'
            echo 'timelapse_filename %Y/%m/%d-timelapse'

            echo
            echo '#------------------------------------------------------------------'
            echo '# Live webcam server'
            echo '#------------------------------------------------------------------'

            # The mini-http server listens to this port for requests (default: 0 = disabled)
            #
            # webcam_port 8081
            #
            # WILL BE SET IN THREAD CONFIGURATION

          # echo "webcam_port ${MOTION_DEF_WEBCAM_PORT}"

            # Quality of the jpeg images produced (default: 50)

            echo 'webcam_quality 50'

            # Output frames at 1 fps when no motion is detected and increase to the
            # rate given by webcam_maxrate when motion is detected (default: off)
            #
            # webcam_motion off

            if [ "${MOTION_DEF_WEBCAM_MOTION}" = "yes" ]
            then
                echo 'webcam_motion on'
            else
                echo 'webcam_motion off'
            fi

            # Maximum framerate for webcam streams (default: 1)
            #
            # webcam_maxrate 1

            echo "webcam_maxrate ${MOTION_DEF_WEBCAM_FRAMERATE}"

            # Restrict webcam connections to localhost only (default: on)

            echo 'webcam_localhost off'

            # Limits the number of images per connection (default: 0 = unlimited)
            # Number can be defined by multiplying actual webcam rate by desired number of seconds
            # Actual webcam rate is the smallest of the numbers framerate and webcam_maxrate

            echo 'webcam_limit 0'

            echo
            echo '#------------------------------------------------------------------'
            echo '# HTTP based control'
            echo '#------------------------------------------------------------------'
            echo

            # TCP/IP port for the http server to listen on (default: 0 = disabled)

            echo "control_port ${MOTION_CONTROL_PORT}"

            # Restrict control connections to localhost only (default: on)

            echo 'control_localhost off'

            # Output for http server, select off to choose raw text plain (default: on)

            echo 'control_html_output on'

            # Authentication for the http based control. Syntax username:password
            # Default: not defined (Disabled)

            if [ "${MOTION_CONTROL_USER}" != "" -a "${MOTION_CONTROL_PASS}" != "" ]
            then
                echo "control_authentication ${MOTION_CONTROL_USER}:${MOTION_CONTROL_PASS}"
            else
                echo '; control_authentication username:password'
            fi

            echo
            echo '#------------------------------------------------------------------'
            echo '# Tracking (Pan/Tilt)'
            echo '#------------------------------------------------------------------'

            # Type of tracker (0=none (default), 1=stepper, 2=iomojo, 3=pwc, 4=generic, 5=uvcvideo)
            # The generic type enables the definition of motion center and motion size to
            # be used with the conversion specifiers for options like on_motion_detected

            echo 'track_type 0'

            # Enable auto tracking (default: off)

            echo 'track_auto off'

            # Serial port of motor (default: none)

          # echo '; track_port value'

            # Motor number for x-axis (default: -1)

            echo 'track_motorx -1'

            # Motor number for y-axis (default: -1)

            echo 'track_motory -1'

            # Maximum value on x-axis (default: 0)

            echo 'track_maxx 0'

            # Maximum value on y-axis (default: 0)

            echo 'track_maxy 0'

            # ID of an iomojo camera if used (default: 0)

            echo 'track_iomojo_id 0'

            # Angle in degrees the camera moves per step on the X-axis
            # with auto-track (default: 10)
            # Currently only used with pwc type cameras

            echo 'track_step_angle_x 10'

            # Angle in degrees the camera moves per step on the Y-axis
            # with auto-track (default: 10)
            # Currently only used with pwc type cameras

            echo 'track_step_angle_y 10'

            # Delay to wait for after tracking movement as number
            # of picture frames (default: 10)

            echo 'track_move_wait 10'

            # Speed to set the motor to (default: 255)

            echo 'track_speed 255'

            # Number of steps to make (default: 40)

            echo 'track_stepsize 40'

            echo
            echo '#------------------------------------------------------------------'
            echo '# External Commands, Warnings and Logging:'
            echo '#------------------------------------------------------------------'

            # You can use conversion specifiers for the on_xxxx commands'
            # %Y = year, %m = month, %d = date,'
            # %H = hour, %M = minute, %S = second,'
            # %v = event, %q = frame number, %t = thread (camera) number,'
            # %D = changed pixels, %N = noise level,'
            # %i and %J = width and height of motion area,'
            # %K and %L = X and Y coordinates of motion center'
            # %C = value defined by text_event'
            # %f = filename with full path'
            # %n = number indicating filetype'
            # Both %f and %n are only defined for on_picture_save,'
            # on_movie_start and on_movie_end'
            # Quotation marks round string are allowed.'

            # Do not sound beeps when detecting motion (default: on)
            # Note: Motion never beeps when running in daemon mode.

            echo 'quiet on'

            # Command to be executed when motion in a predefined area is detected
            # Check option 'area_detect'. (default: none)
            #
            # ; on_area_detected value

            # Command to be executed when a camera can't be opened or if it is lost
            # NOTE: There is situations when motion don't detect a lost camera!
            # It depends on the driver, some drivers dosn't detect a lost camera at all
            # Some hangs the motion thread. Some even hangs the PC! (default: none)
            #
            # ; on_camera_lost value

            if [ "${MOTION_DEF_ON_CAMERALOST}" != "" ]
            then
                exec_str=`add_script_path "${MOTION_DEF_ON_CAMERALOST}"`

                if `check_script_exists_and_executable "${exec_str}"`
                then
                    echo "on_camera_lost ${exec_str}"
                else
                    # error
                    write_to_config_log -error "Parameter MOTION_DEF_ON_CAMERALOST='${MOTION_DEF_ON_CAMERALOST}'"
                    write_to_config_log -error -ff "has been set but at the file doesn't exist or has no execute rights!"
                fi
            fi

            # Command to be executed when an event starts. (default: none)
            # An event starts at first motion detected after a period of no motion defined by gap 
            #
            # ; on_event_start value

            if [ "${MOTION_DEF_ON_EVENTSTART}" != "" ]
            then
                exec_str=`add_script_path "${MOTION_DEF_ON_EVENTSTART}"`

                if `check_script_exists_and_executable "${exec_str}"`
                then
                    echo "on_event_start ${exec_str}"
                else
                    # error
                    write_to_config_log -error "Parameter MOTION_DEF_ON_EVENTSTART='${MOTION_DEF_ON_EVENTSTART}'"
                    write_to_config_log -error -ff "has been set but the file doesn't exist or has no execute rights!"
                fi
            fi

            # Command to be executed when an event ends after a period of no motion
            # (default: none). The period of no motion is defined by option gap.
            #
            # on_event_end value

            if [ "${MOTION_DEF_ON_EVENTEND}" != "" ]
            then
                exec_str=`add_script_path "${MOTION_DEF_ON_EVENTEND}"`

                if `check_script_exists_and_executable "${exec_str}"`
                then
                    echo "on_event_end ${exec_str}"
                else
                    # error
                    write_to_config_log -error "Parameter MOTION_DEF_ON_EVENTEND='${MOTION_DEF_ON_EVENTEND}'"
                    write_to_config_log -error -ff "has been set but the file doesn't exist or has no execute rights!"
                fi
            fi

            # Command to be executed when a motion frame is detected (default: none)
            #
            # ; on_motion_detected value

            if [ "${MOTION_DEF_ON_MOTIONDETECT}" != "" ]
            then
                exec_str=`add_script_path "${MOTION_DEF_ON_MOTIONDETECT}"`

                if `check_script_exists_and_executable "${exec_str}"`
                then
                    echo "on_motion_detected ${exec_str}"
                else
                    # error
                    write_to_config_log -error "Parameter MOTION_DEF_ON_MOTIONDETECT='${MOTION_DEF_ON_MOTIONDETECT}'"
                    write_to_config_log -error -ff "has been set but the file doesn't exist or has no execute rights!"
                fi
            fi

            # Command to be executed when a movie file (.mpg|.avi) is created. (default: none)
            # To give the filename as an argument to a command append it with %f
            #
            # ; on_movie_start value

            if [ "${MOTION_DEF_ON_MOVIESTART}" != "" ]
            then
                exec_str=`add_script_path "${MOTION_DEF_ON_MOVIESTART}"`

                if `check_script_exists_and_executable "${exec_str}"`
                then
                    echo "on_movie_start ${exec_str}"
                else
                    # error
                    write_to_config_log -error "Parameter MOTION_DEF_ON_MOVIESTART='${MOTION_DEF_ON_MOVIESTART}'"
                    write_to_config_log -error -ff "has been set but the file doesn't exist or has no execute rights!"
                fi
            fi

            # Command to be executed when a movie file (.mpg|.avi) is closed. (default: none)
            # To give the filename as an argument to a command append it with %f
            #
            # ; on_movie_end value

            if [ "${MOTION_DEF_ON_MOVIEEND}" != "" ]
            then
                exec_str=`add_script_path "${MOTION_DEF_ON_MOVIEEND}"`

                if `check_script_exists_and_executable "${exec_str}"`
                then
                    if [ "${MOTION_SQL_ENABLED}" = "yes" ]
                    then
                        # update file size in sql database
                        echo "on_movie_end ${motionconfig_path}/motion_update_db_fields.sh %f;${exec_str}"
                    else
                        echo "on_movie_end ${exec_str}"
                    fi
                else
                    # error
                    write_to_config_log -error "Parameter MOTION_DEF_ON_MOVIEEND='${MOTION_DEF_ON_MOVIEEND}'"
                    write_to_config_log -error -ff "has been set but the file doesn't exist or has no execute rights!"
                fi
            fi

            # Command to be executed when a picture (.ppm|.jpg) is saved (default: none)
            # To give the filename as an argument to a command append it with %f
            #
            # ; on_picture_save value

            if [ "${MOTION_DEF_ON_PICTURESAVE}" != "" ]
            then
                exec_str=`add_script_path "${MOTION_DEF_ON_PICTURESAVE}"`

                if `check_script_exists_and_executable "${exec_str}"`
                then
                    if [ "${MOTION_SQL_ENABLED}" = "yes" ]
                    then
                        # update file size in sql database
                        echo "on_picture_save ${motionconfig_path}/motion_update_db_fields.sh %f;${exec_str}"
                    else
                        echo "on_picture_save ${exec_str}"
                    fi
                else
                    # error
                    write_to_config_log -error "Parameter MOTION_DEF_ON_PICTURESAVE='${MOTION_DEF_ON_PICTURESAVE}'"
                    write_to_config_log -error -ff "has been set but the file doesn't exist or has no execute rights!"
                fi
            fi

            if [ "${MOTION_SQL_ENABLED}" = "yes" ]
            then
                echo
                echo '#------------------------------------------------------------------'
                echo '# Common Options For MySQL and PostgreSQL database features.'
                echo '# Options require the MySQL/PostgreSQL options to be active also.'
                echo '#------------------------------------------------------------------'

                # pre_capture 8                 - default: 0
                # post_capture 8                - default: 0
                # output_all off
                # output_normal best (or) first - default: on
                # output_motion off             - default: off
                # text_event %Y%m%d%H%M%S
                # ffmpeg_cap_new on             - default: off
                # ffmpeg_video_codec msmpeg4
                # sql_log_snapshot off

                # Log to the database when creating motion triggered image file  (default: on)
                # ; sql_log_image on

                if [ "${MOTION_SQL_DEF_LOG_IMAGE}" != "yes" ]
                then
                    echo "sql_log_image off"
                else
                    echo "sql_log_image on"
                fi

                # Log to the database when creating a snapshot image file (default: on)
                # ;  sql_log_snapshot on

                if [ "${MOTION_SQL_DEF_LOG_SNAPSHOT}" != "yes" ]
                then
                    echo "sql_log_snapshot off"
                else
                    echo "sql_log_snapshot on"
                fi

                # Log to the database when creating motion triggered mpeg file (default: off)
                # ; sql_log_mpeg off

                if [ "${MOTION_SQL_DEF_LOG_MOVIE}" = "yes" ]
                then
                    echo "sql_log_mpeg on"
                else
                    echo "sql_log_mpeg off"
                fi

                # Log to the database when creating timelapse mpeg file (default: off)
                # ; sql_log_timelapse off

                if [ "${MOTION_SQL_DEF_LOG_TIMELAPS}" = "yes" ]
                then
                    echo "sql_log_timelapse on"
                else
                    echo "sql_log_timelapse off"
                fi

                # SQL query string that is sent to the database
                # Use same conversion specifiers has for text features. Additional special
                # conversion specifiers are:
                #
                # %t - thread (camera) number 
                # %f - filename with full path
                # %q - frame number
                # %n - the number representing the file_type
                #       1 - image
                #       2 - image snapshot
                #       4 - image motion
                #       8 - mpeg
                #      16 - mpeg motion
                #      32 - mpeg timelaps
                # %Y-%m-%d %T - time stamp, e.g. 2011-12-05 17:10:03
                # %C - value defined by text_event, default: %Y%m%d%H%M%S
                #
                # Default: insert into security(camera, filename, frame, file_type, time_stamp, text_event)
                #                               values('%t', '%f', '%q', '%n', '%Y-%m-%d %T', '%C')
                echo "sql_query INSERT IGNORE INTO ${MOTION_SQL_TABLE}(camera, filename, frame, file_type, time_stamp, event_time_stamp) VALUES('%t', '%f', '%q', '%n', '%Y-%m-%d %T', '%C')"

                echo
                echo '#------------------------------------------------------------------'
                echo '# Database Options For MySQL'
                echo '#------------------------------------------------------------------'

                # Mysql database to log to (default: not defined)
                # ; mysql_db value

                echo "mysql_db ${MOTION_SQL_DATABASE}"

                # The host on which the database is located (default: localhost)
                # ; mysql_host value
                echo "mysql_host ${MOTION_SQL_HOST}"

                # User account name for MySQL database (default: not defined)
                # ; mysql_user value
                echo "mysql_user ${MOTION_SQL_USER}"

                # User password for MySQL database (default: not defined)
                # ; mysql_password value
                echo "mysql_password ${MOTION_SQL_PASS}"
            fi

          # echo
          # echo '#------------------------------------------------------------------'
          # echo '# Database Options For PostgreSQL'
          # echo '#------------------------------------------------------------------'

          # # PostgreSQL database to log to (default: not defined)

          # echo '; pgsql_db value'

          # # The host on which the database is located (default: not defined)

          # echo '; pgsql_host value'

          # # User account name for PostgreSQL database (default: not defined)

          # echo '; pgsql_user value'

          # # User password for PostgreSQL database (default: not defined)

          # echo '; pgsql_password value'

          # # Port on which the PostgreSQL database is located (default: 5432)

          # echo '; pgsql_port 5432'

          # echo
          # echo '#------------------------------------------------------------------'
          # echo '# Video Loopback Device (vloopback project)'
          # echo '#------------------------------------------------------------------'

          # # Output images to a video4linux loopback device
          # # The value '-' means next available (default: not defined)

          # echo '; video_pipe value'

          # # Output motion images to a video4linux loopback device
          # # The value '-' means next available (default: not defined)

          # echo '; motion_video_pipe value'

            echo
            echo '#------------------------------------------------------------------'
            echo '# Thread config files - One for each camera.'
            echo '# Except if only one camera - You only need this config file.'
            echo '# If you have more than one camera you MUST define one thread'
            echo '# config file for each camera in addition to this config file.'
            echo '#------------------------------------------------------------------'

            # Remember: If you have more than one camera you must have one
            # thread file for each camera. E.g. 2 cameras requires 3 files:
            # This motion.conf file AND thread1.conf and thread2.conf.
            # Only put the options that are unique to each camera in the
            # thread config files. 

            idx=1
            while [ ${idx} -le ${MOTION_N} ]
            do
                eval active='$MOTION_'${idx}'_ACTIVE'

                if [ "${active}" = "yes" ]
                then
                    echo "thread ${generate_threadconf}${idx}.conf"
                fi

                idx=`expr ${idx} + 1`
            done

        } > ${generate_motionconf}
    fi
}

#----------------------------------------------------------------------------------------
# creating motion thread configuration
# $1 - instance number
#----------------------------------------------------------------------------------------
create_motion_thread_config ()
{
    tidx=${1}

    mecho "creating motion thread configuration file (${tidx}) ..."

    {
        eval m_video_text_left='$MOTION_'${tidx}'_TEXT_LEFT'
        eval m_video_text_right='$MOTION_'${tidx}'_TEXT_RIGHT'
        eval m_video_text_size='$MOTION_'${tidx}'_TEXT_SIZE'

        eval m_video_device='$MOTION_'${tidx}'_VIDEO_DEVICE'
        eval m_video_input='$MOTION_'${tidx}'_VIDEO_INPUT'

        eval m_netcam_url='$MOTION_'${tidx}'_NETCAM_URL'
        eval m_netcam_user='$MOTION_'${tidx}'_NETCAM_USER'
        eval m_netcam_pass='$MOTION_'${tidx}'_NETCAM_PASS'

        eval m_video_freq='$MOTION_'${tidx}'_VIDEO_FREQ'
        eval m_video_rotate='$MOTION_'${tidx}'_VIDEO_ROTATE'
        eval m_video_width='$MOTION_'${tidx}'_VIDEO_WIDTH'
        eval m_video_height='$MOTION_'${tidx}'_VIDEO_HEIGHT'
        eval m_video_framerate='$MOTION_'${tidx}'_VIDEO_FRAMERATE'
        eval m_video_frametime='$MOTION_'${tidx}'_VIDEO_FRAMETIME'
        eval m_video_brightness='$MOTION_'${tidx}'_VIDEO_BRIGHTNESS'
        eval m_video_contrast='$MOTION_'${tidx}'_VIDEO_CONTRAST'
        eval m_video_saturation='$MOTION_'${tidx}'_VIDEO_SATURATION'

        eval m_motion_trigger='$MOTION_'${tidx}'_MOTION_TRIGGER'
      # eval m_motion_trigger_tune='$MOTION_'${tidx}'_MOTION_TRIGGER_TUNE'
        eval m_motion_noise_level='$MOTION_'${tidx}'_MOTION_NOISE_LEVEL'
        eval m_motion_noise_tune='$MOTION_'${tidx}'_NOISE_TUNE'
        eval m_motion_lightswitch='$MOTION_'${tidx}'_MOTION_LIGHTSWITCH'
        eval m_motion_frames='$MOTION_'${tidx}'_MOTION_FRAMES'
        eval m_motion_precapture='$MOTION_'${tidx}'_MOTION_PRECAPTURE'
        eval m_motion_postcapture='$MOTION_'${tidx}'_MOTION_POSTCAPTURE'
        eval m_motion_endgap='$MOTION_'${tidx}'_MOTION_ENDGAP'
        eval m_motion_locate='$MOTION_'${tidx}'_MOTION_LOCATE'

        eval m_mask_file='$MOTION_'${tidx}'_MASK_FILE'
        eval m_mask_speed='$MOTION_'${tidx}'_MASK_SPEED'

        eval m_capture_image='$MOTION_'${tidx}'_CAPTURE_IMAGE'
        eval m_capture_all='$MOTION_'${tidx}'_CAPTURE_ALL'
        eval m_capture_snapshot='$MOTION_'${tidx}'_CAPTURE_SNAPSHOT'
        eval m_capture_quality='$MOTION_'${tidx}'_CAPTURE_QUALITY'
        eval m_capture_movie='$MOTION_'${tidx}'_CAPTURE_MOVIE'
        eval m_capture_length='$MOTION_'${tidx}'_CAPTURE_LENGTH'
        eval m_capture_timelaps='$MOTION_'${tidx}'_CAPTURE_TIMELAPS'
        eval m_capture_codec='$MOTION_'${tidx}'_CAPTURE_CODEC'

        m_target_path=${MOTION_DEF_TARGET_PATH}/camera-${tidx}

        eval m_webcam_port='$MOTION_'${tidx}'_WEBCAM_PORT'
        eval m_webcam_motion='$MOTION_'${tidx}'_WEBCAM_MOTION'
        eval m_webcam_framerate='$MOTION_'${tidx}'_WEBCAM_FRAMERATE'

        eval m_on_cameralost='$MOTION_'${tidx}'_ON_CAMERALOST'
        eval m_on_eventstart='$MOTION_'${tidx}'_ON_EVENTSTART'
        eval m_on_eventend='$MOTION_'${tidx}'_ON_EVENTEND'
        eval m_on_motiondetect='$MOTION_'${tidx}'_ON_MOTIONDETECT'
        eval m_on_moviestart='$MOTION_'${tidx}'_ON_MOVIESTART'
        eval m_on_movieend='$MOTION_'${tidx}'_ON_MOVIEEND'
        eval m_on_picturesave='$MOTION_'${tidx}'_ON_PICTURESAVE'

        #----------------------------------------------------------------------------------------
        print_short_header "${generate_threadconf}${tidx}.conf" "${pgmname}" "${motion_version}"
        #----------------------------------------------------------------------------------------

        #----------------------------------------------------------------------------------------
        # Capture device options
        #----------------------------------------------------------------------------------------

        check_videodevice ${m_video_device}

        if [ "${m_netcam_url}" = "" ]
        then
            echo "videodevice ${m_video_device}"
            echo "input ${m_video_input}"

            if [ "${m_video_input}" = "" -o "${m_video_input}" = "8" ]
            then
                if [ "${EISFAIR_SYSTEM}" = "eisfair-1" ]
                then
                    # check USB activation only on eisfair-1 because eisfair-2
                    # uses an autoload mechanism
                    if ! check_active_usb
                    then
                        write_to_config_log -warn     "The parameter MOTION_${tidx}_VIDEO_INPUT hasn't been set or"
                        write_to_config_log -warn -ff "a value of '8', although START_USB='no' has been set. Your"
                        write_to_config_log -warn -ff "USB-device will most likely not work!"
                    fi
                fi
            fi
        else
            write_to_config_log -warn     "Parameter MOTION_${tidx}_VIDEO_DEVICE will be ignored"
            write_to_config_log -warn -ff "because MOTION_${tidx}_NETCAM_URL has been set!"
        fi

        #----------------------------------------------------------------------------------------
        # Netcam settings
        #----------------------------------------------------------------------------------------

        if [ "${m_netcam_url}" != "" ]
        then
            # example url for Axis webcam
            #
            # ; netcam_url http://192.168.1.52/mjpg/video.mjpg
            #   netcam_url http://192.168.1.52/axis-cgi/mjpg/video.cgi
            # ; netcam_url http://192.168.1.52/axis-cgi/mjpg/video.cgi?showlength=1
            # ; netcam_url http://192.168.1.52/axis-cgi/mjpg/video.cgi?resolution=320x240
            # ; netcam_url http://192.168.1.52/axis-cgi/mjpg/video.cgi?resolution=640x480

            echo "netcam_url ${m_netcam_url}"

            if [ "${m_netcam_user}" != "" -a "${m_netcam_pass}" != "" ]
            then
                echo "netcam_userpass ${m_netcam_user}:${m_netcam_pass}"
          # else
          #     echo '; netcam_userpass username:password'
            fi

        fi

        # The frequency to set the tuner to (kHz) (only for TV tuner cards) (default: 0)

        if [ "${m_video_freq}" != "" -a "${m_video_freq}" != "${MOTION_DEF_VIDEO_FREQ}" ]
        then
            echo "frequency ${m_video_freq}"
        fi

        # Rotate image this number of degrees. The rotation affects all saved images as
        # well as mpeg movies. Valid values: 0 (default = no rotation), 90, 180 and 270.

        if [ "${m_video_rotate}" != "" -a "${m_video_rotate}" != "${MOTION_DEF_VIDEO_ROTATE}" ]
        then
            echo "rotate ${m_video_rotate}"
        fi

        # Image width (pixels). Valid range: Camera dependent, default: 352

        if [ "${m_video_width}" != "" ]
        then
            echo "width ${m_video_width}"

            if [ `expr ${m_video_width} % 16` -ne 0 ]
            then
                write_to_config_log -error "Parameter MOTION_${tidx}_VIDEO_WIDTH='${m_video_width}' has been set"
                write_to_config_log -error -ff "to a non modulo 16 value."
            fi
        fi

        # Image height (pixels). Valid range: Camera dependent, default: 288

        if [ "${m_video_height}" != "" ]
        then
            echo "height ${m_video_height}"

            if [ `expr ${m_video_height} % 16` -ne 0 ]
            then
                write_to_config_log -error "Parameter MOTION_${tidx}_VIDEO_HEIGHT='${m_video_height}' has been set"
                write_to_config_log -error -ff "to a non modulo 16 value."
            fi
        fi

        # Maximum number of frames to be captured per second.
        # Valid range: 2-100. Default: 100 (almost no limit).

        if [ "${m_video_framerate}" != "" -a "${m_video_framerate}" != "${MOTION_DEF_VIDEO_FRAMERATE}" ]
        then
            echo "framerate ${m_video_framerate}"
        fi

        # Minimum time in seconds between capturing picture frames from the camera.
        # Default: 0 = disabled - the capture rate is given by the camera framerate.
        # This option is used when you want to capture images at a rate lower than
        # 2 per second.

        if [ "${m_video_frametime}" != "" -a "${m_video_frametime}" != "${MOTION_DEF_VIDEO_FRAMETIME}" ]
        then
            echo "minimum_frame_time ${m_video_frametime}"
        fi

        # Let motion regulate the brightness of a video device (default: off).
        # The auto_brightness feature uses the brightness option as its target value.
        # If brightness is zero auto_brightness will adjust to average brightness
        # value 128. Only recommended for cameras without auto brightness

        # Set the initial brightness of a video device.
        # If auto_brightness is enabled, this value defines the average brightness level
        # which Motion will try and adjust to.
        # Valid range 0-255, default 0 = disabled

        if [ "${m_video_brightness}" != "" -a "${m_video_brightness}" != "${MOTION_DEF_VIDEO_BRIGHTNESS}" ]
        then
            echo "${m_video_brightness}" | grep -q "auto:"

            if [ $? -eq 0 ]
            then
                # multiple parameter given
                echo 'auto_brightness on'
                echo "brightness `echo ${m_video_brightness} | cut -d: -f2`"
            else
                echo 'auto_brightness off'
                echo "brightness ${m_video_brightness}"
            fi
        fi

        # Set the contrast of a video device.
        # Valid range 0-255, default 0 = disabled

        if [ "${m_video_contrast}" != "" -a "${m_video_contrast}" != "${MOTION_DEF_VIDEO_CONTRAST}" ]
        then
            echo "contrast ${m_video_contrast}"
        fi

        # Set the saturation of a video device.
        # Valid range 0-255, default 0 = disabled

        if [ "${m_video_saturation}" != "" -a "${m_video_saturation}" != "${MOTION_DEF_VIDEO_SATURATION}" ]
        then
            echo "saturation ${m_video_saturation}"
        fi

        #----------------------------------------------------------------------------------------
        # Motion detection settings
        #----------------------------------------------------------------------------------------

        # Threshold for number of changed pixels in an image that
        # triggers motion detection (default: 1500)

        if [ "${m_motion_trigger}" != "" -a "${m_motion_trigger}" != "${MOTION_DEF_MOTION_TRIGGER}" ]
        then
            echo "threshold ${m_motion_trigger}"
        fi

        # Automatically tune the threshold down if possible (default: off)
        #  Better don't enable this option it's broken  -- AngelCarpintero - 07 May 2008 

      # if [ "${m_motion_trigger_tune}" != "" -a "${m_motion_trigger_tune}" != "${MOTION_DEF_MOTION_TRIGGER_TUNE}" ]
      # then
      #     if [ "${m_motion_trigger_tune}" = "yes" ]
      #     then
      #         echo 'threshold_tune on'
      #     else
      #         echo 'threshold_tune off'
      #     fi
      # fi


        # Automatically tune the noise threshold (default: on)
        # Noise threshold for the motion detection (default: 32)

        if [ "${m_motion_noise_tune}" != "" -a "${m_motion_noise_tune}" != "${MOTION_DEF_MOTION_NOISE_TUNE}" ]
        then
            if [ "${m_motion_noise_tune}" = "no" ]
            then
                echo 'noise_tune off'

                if [ "${m_motion_noise_level}" != "" -a "${m_motion_noise_level}" != "${MOTION_DEF_MOTION_NOISE_LEVEL}" ]
                then
                    echo "noise_level ${m_motion_noise_level}"
                fi
            else
                # parameter has been set to 'yes' therefore the noise_level
                # value is ignored
                echo 'noise_tune on'
            fi
        fi

        # PGM file to use as a sensitivity mask.
        # Full path name to. (Default: not defined)

        if [ "${m_mask_file}" != "" -a "${m_mask_file}" != "${MOTION_DEF_MASK_FILE}" ]
        then
            if [ -f ${motionconfig_path}/${m_mask_file} ]
            then
                echo "mask_file ${motionconfig_path}/${m_mask_file}"
            else
                # file doesn't exist
                write_to_config_log -warn "Parameter MOTION_${tidx}_MASK_FILE='${m_mask_file}'"
                write_to_config_log -warn -ff "has been set but the file doesn't exist!"
            fi
        fi

        # Dynamically create a mask file during operation (default: 0)
        # Adjust speed of mask changes from 0 (off) to 10 (fast)

        if [ "${m_mask_speed}" != "" -a "${m_mask_speed}" != "${MOTION_DEF_MASK_SPEED}" ]
        then
            echo "smart_mask_speed ${m_mask_speed}"
        fi

        # Ignore sudden massive light intensity changes given as a percentage of the picture
        # area that changed intensity. Valid range: 0 - 100 , default: 0 = disabled

        if [ "${m_motion_lightswitch}" != "" -a "${m_motion_lightswitch}" != "${MOTION_DEF_MOTION_LIGHTSWITCH}" ]
        then
            echo "lightswitch ${m_motion_lightswitch}"
        fi

        # Picture frames must contain motion at least the specified number of frames
        # in a row before they are detected as true motion. At the default of 1, all
        # motion is detected. Valid range: 1 to thousands, recommended 1-10

        if [ "${m_motion_frames}" != "" -a "${m_motion_frames}" != "${MOTION_DEF_MOTION_FRAMES}" ]
        then
            echo "minimum_motion_frames ${m_motion_frames}"
        fi

        # Specifies the number of pre-captured (buffered) pictures from before motion
        # was detected that will be output at motion detection.
        # Recommended range: 0 to 5 (default: 0)

        if [ "${m_motion_precapture}" != "" -a "${m_motion_precapture}" != "${MOTION_DEF_MOTION_PRECAPTURE}" ]
        then
            echo "pre_capture ${m_motion_precapture}"
        fi

        # Number of frames to capture after motion is no longer detected (default: 0)

        if [ "${m_motion_postcapture}" != "" -a "${m_motion_postcapture}" != "${MOTION_DEF_MOTION_POSTCAPTURE}" ]
        then
            echo "post_capture ${m_motion_postcapture}"
        fi

        # Gap is the seconds of no motion detection that triggers the end of an event
        # An event is defined as a series of motion images taken within a short timeframe.
        # Recommended value is 60 seconds (Default). The value 0 is allowed and disables
        # events causing all Motion to be written to one single mpeg file and no pre_capture.

        if [ "${m_motion_endgap}" != "" -a "${m_motion_endgap}" != "${MOTION_DEF_MOTION_ENDGAP}" ]
        then
            echo "gap ${m_motion_endgap}"
        fi

        # Maximum length in seconds of an mpeg movie
        # When value is exceeded a new mpeg file is created. (Default: 0 = infinite)

        if [ "${m_capture_length}" != "" -a "${m_capture_length}" != "${MOTION_DEF_CAPTURE_LENGTH}" ]
        then
            echo "max_mpeg_time ${m_capture_length}"
        fi

        # Number of frames per second to capture when not detecting
        # motion (saves CPU load) (Default: 0 = disabled)

        # Always save images even if there was no motion (default: off)

        if [ "${m_capture_all}" != "" -a "${m_capture_all}" != "${MOTION_DEF_CAPTURE_ALL}" ]
        then
            if [ "${m_capture_all}" = "yes" ]
            then
                echo 'output_all on'
            else
                echo 'output_all off'
            fi
        fi

        #----------------------------------------------------------------------------------------
        # Image file output
        #----------------------------------------------------------------------------------------

        # Output 'normal' pictures when motion is detected (default: on)
        # Valid values: on, off, first, best
        # When set to 'first', only the first picture of an event is saved.
        # Picture with most motion of an event is saved when set to 'best'.
        # Can be used as preview shot for the corresponding movie.

        if [ "${m_capture_image}" != "" -a "${m_capture_image}" != "${MOTION_DEF_CAPTURE_IMAGE}" ]
        then
            case "${m_capture_image}" in
                'best')
                    echo 'output_normal best'
                    ;;
                'first')
                    echo 'output_normal first'
                    ;;
                'no')
                    echo 'output_normal off'
                    ;;
                *)  # yes
                    echo 'output_normal on'
                    ;;
            esac
        fi

        # The quality (in percent) to be used by the jpeg compression

        if [ "${m_capture_quality}" != "" -a "${m_capture_quality}" != "${MOTION_DEF_CAPTURE_QUALITY}" ]
        then
            echo "quality ${m_capture_quality}"
        fi

        #----------------------------------------------------------------------------------------
        # FFMPEG related options
        #----------------------------------------------------------------------------------------

        if [ "${m_capture_movie}" != "" -a "${m_capture_movie}" != "${MOTION_DEF_CAPTURE_MOVIE}" ]
        then
            if [ "${m_capture_movie}" = "yes" ]
            then
                echo 'ffmpeg_cap_new on'
            else
                echo 'ffmpeg_cap_new off'
            fi
        fi

        if [ "${m_capture_timelaps}" != "" -a "${m_capture_timelaps}" != "${MOTION_DEF_CAPTURE_TIMELAPS}" ]
        then
            echo "${m_capture_timelaps}" | grep -q ":"

            if [ $? -eq 0 ]
            then
                echo "ffmpeg_timelapse `echo ${m_capture_timelaps} | cut -d: -f1`"
                echo "ffmpeg_timelapse_mode `echo ${m_capture_timelaps} | cut -d: -f2`"
            else
                echo "ffmpeg_timelapse ${m_capture_timelaps}"
                echo 'ffmpeg_timelapse_mode daily'
            fi
        fi

        if [ "${m_capture_codec}" != "" -a "${m_capture_codec}" != "${MOTION_DEF_CAPTURE_CODEC}" ]
        then
            echo "ffmpeg_video_codec ${m_capture_codec}"
        fi

        #----------------------------------------------------------------------------------------
        # Snapshots (traditional periodic webcam file output)
        #----------------------------------------------------------------------------------------

        m_singlesnap=0
        if [ "${m_capture_snapshot}" != "" -a  "${m_capture_snapshot}" != "${MOTION_DEF_CAPTURE_SNAPSHOT}" ]
        then
            echo "${m_capture_snapshot}" | grep -q "^single:"

            if [ $? -eq 0 ]
            then
                # only one snapshoot will be saved each interval 
                m_singlesnap=1
                echo "snapshot_interval `echo ${m_capture_snapshot} | cut -d: -f2`"
            else
                echo "snapshot_interval ${m_capture_snapshot}"
            fi
        fi

        # Locate and draw a box around the moving object.
        # Valid values: on, off and preview (default: off)
        # Set to 'preview' will only draw a box in preview_shot pictures.

        if [ "${m_motion_locate}" != "" -a  "${m_motion_locate}" != "${MOTION_DEF_MOTION_LOCATE}" ]
        then
            case "${m_motion_locate}" in
                'yes')
                    echo "locate on"
                    ;;
                'no')
                    echo "locate off"
                    ;;
                'preview')
                    echo "locate preview"
                    ;;
            esac
        fi

        # Draw a user defined text on the images using same options as C function strftime(3)
        # Default: Not defined = no text
        # Text is placed in lower left corner

        if [ "${m_video_text_left}" != "" -a \
             "${m_video_text_left}" != "." -a \
             "${m_video_text_left}" != "${MOTION_DEF_TEXT_LEFT}" ]
        then
            echo "text_left  ${m_video_text_left}"
        fi

        # Text is placed in lower right corner

        if [ "${m_video_text_right}" != "" -a \
             "${m_video_text_right}" != "." -a \
             "${m_video_text_right}" != "${MOTION_DEF_TEXT_RIGHT}" ]
        then
            echo "text_right ${m_video_text_right}"
        fi

        # Draw characters at twice normal size on images. (default: off)

        if [ "${m_video_text_size}" != "" -a \
             "${m_video_text_size}" != "${MOTION_DEF_TEXT_SIZE}" ]
        then
            if [ "${m_video_text_size}" = "double" ]
            then
                echo 'text_double on'
            else
                echo 'text_double off'
            fi
        fi

        #----------------------------------------------------------------------------------------
        # Target Directories and filenames For Images And Films
        #----------------------------------------------------------------------------------------

        # Target base directory for pictures and films
        # Recommended to use absolute path. (Default: current working directory)

        # use individual directory
        if [ ! -d ${m_target_path} ]
        then
            # create target directory
            mkdir -p ${m_target_path}

            chown -R ${motion_apache_user}  ${m_target_path}
            chgrp -R ${motion_apache_group} ${m_target_path}
        fi

        echo "target_dir ${m_target_path}"

        # File path for snapshots (jpeg or ppm) relative to target_dir
        # Default: %v-%Y%m%d%H%M%S-snapshot
        # Default value is equivalent to legacy oldlayout option
        # For Motion 3.0 compatible mode choose: %Y/%m/%d/%H/%M/%S-snapshot
        # File extension .jpg or .ppm is automatically added so do not include this.
        # Note: A symbolic link called lastsnap.jpg created in the target_dir will always
        # point to the latest snapshot, unless snapshot_filename is exactly 'lastsnap'

        if [ ${m_singlesnap} -eq 1 ]
        then
            # only one snapshoot will be saved each interval 
            echo 'snapshot_filename lastsnap'
        fi

        #----------------------------------------------------------------------------------------
        # Live webcam server
        #----------------------------------------------------------------------------------------

        # The mini-http server listens to this port for requests (default: 0 = disabled)

        m_uniq_webcam_port="`get_uniq_webcamport ${tidx}`"

        if [ "${m_webcam_port}" != "${m_uniq_webcam_port}" ]
        then
            if [ "${m_webcam_port}" = "" ]
            then
                eval "MOTION_${tidx}_WEBCAM_PORT=${m_uniq_webcam_port}"
                write_to_config_log -info "Parameter MOTION_${tidx}_WEBCAM_PORT='${m_uniq_webcam_port}' has been set!"
            else
                write_to_config_log -error "Parameter MOTION_${tidx}_WEBCAM_PORT='${m_webcam_port}' is not uniq!"
                write_to_config_log -error -ff "Please change its value to '${m_uniq_webcam_port}'."
            fi
        fi

        echo "webcam_port ${m_uniq_webcam_port}"

        # export port for later web configuration use
        eval "MOTION_${tidx}_WEBCAM_PORT='${m_uniq_webcam_port}'"

        # Output frames at 1 fps when no motion is detected and increase to the
        # rate given by webcam_maxrate when motion is detected (default: off)

        if [ "${m_webcam_motion}" != "" -a "${m_webcam_motion}" != "${MOTION_DEF_WEBCAM_MOTION}" ]
        then
            if [ "${m_webcam_motion}" = "yes" ]
            then
                echo 'webcam_motion on'
            else
                echo 'webcam_motion off'
            fi
        fi

        # Maximum framerate for webcam streams (default: 1)

        if [ "${m_webcam_framerate}" != "" -a "${m_webcam_framerate}" != "${MOTION_DEF_WEBCAM_FRAMERATE}" ]
        then
            echo "webcam_maxrate ${m_webcam_framerate}"
        fi

        #----------------------------------------------------------------------------------------
        # External Commands, Warnings and Logging
        #----------------------------------------------------------------------------------------

        # Command to be executed when a camera can't be opened or if it is lost

        if [ "${m_on_cameralost}" != "" -a "${m_on_cameralost}" != "${MOTION_DEF_ON_CAMERALOST}" ]
        then
            exec_str=`add_script_path "${m_on_cameralost}"`

            if `check_script_exists_and_executable "${exec_str}"`
            then
                echo "on_camera_lost ${exec_str}"
            else
                # error
                write_to_config_log -error "Parameter MOTION_${tidx}_ON_CAMERALOST='${m_on_cameralost}'"
                write_to_config_log -error -ff "has been set but the file doesn't exist or has no execute rights!"
            fi
        fi

        # Command to be executed when an event starts. (default: none)
        # An event starts at first motion detected after a period of no motion defined by gap 

        if [ "${m_on_eventstart}" != "" -a "${m_on_eventstart}" != "${MOTION_DEF_ON_EVENTSTART}" ]
        then
            exec_str=`add_script_path "${m_on_eventstart}"`

            if `check_script_exists_and_executable "${exec_str}"`
            then
                echo "on_event_start ${exec_str}}"
            else
                # error
                write_to_config_log -error "Parameter MOTION_${tidx}_ON_EVENTSTART='${m_on_eventstart}'"
                write_to_config_log -error -ff "has been set but the file doesn't exist or has no execute rights!"
            fi
        fi

        # Command to be executed when an event ends after a period of no motion
        # (default: none). The period of no motion is defined by option gap.

        if [ "${m_on_eventend}" != "" -a "${m_on_eventend}" != "${MOTION_DEF_ON_EVENTEND}" ]
        then
            exec_str=`add_script_path "${m_on_eventend}"`

            if `check_script_exists_and_executable "${exec_str}"`
            then
                echo "on_event_end ${exec_str}"
            else
                # error
                write_to_config_log -error "Parameter MOTION_${tidx}_ON_EVENTEND='${m_on_eventend}'"
                write_to_config_log -error -ff "has been set but the file doesn't exist or has no execute rights!"
            fi
        fi

        # Command to be executed when a motion frame is detected (default: none)

        if [ "${m_on_motiondetect}" != "" -a "${m_on_motiondetect}" != "${MOTION_DEF_ON_MOTIONDETECT}" ]
        then
            exec_str=`add_script_path "${m_on_motiondetect}"`

            if `check_script_exists_and_executable "${exec_str}"`
            then
                echo "on_motion_detected ${exec_str}"
            else
                # error
                write_to_config_log -error "Parameter MOTION_${tidx}_ON_MOTIONDETECT='${m_on_motiondetect}'"
                write_to_config_log -error -ff "has been set but the file doesn't exist or has no execute rights!"
            fi
        fi

        # Command to be executed when a movie file (.mpg|.avi) is created. (default: none)
        # To give the filename as an argument to a command append it with %f

        if [ "${m_on_moviestart}" != "" -a "${m_on_moviestart}" != "${MOTION_DEF_ON_MOVIESTART}" ]
        then
            exec_str=`add_script_path "${m_on_moviestart}"`

            if `check_script_exists_and_executable "${exec_str}"`
            then
                echo "on_movie_start ${exec_str}"
            else
                # error
                write_to_config_log -error "Parameter MOTION_${tidx}_ON_MOVIESTART='${m_on_moviestart}'"
                write_to_config_log -error -ff "has been set but the file doesn't exist or has no execute rights!"
            fi
        fi

        # Command to be executed when a movie file (.mpg|.avi) is closed. (default: none)
        # To give the filename as an argument to a command append it with %f

        if [ "${m_on_movieend}" != "" -a "${m_on_movieend}" != "${MOTION_DEF_ON_MOVIEEND}" ]
        then
            exec_str=`add_script_path "${m_on_movieend}"`

            if `check_script_exists_and_executable "${exec_str}"`
            then
                if [ "${MOTION_SQL_ENABLED}" = "yes" ]
                then
                    # update file size in sql database
                    echo "on_movie_end ${motionconfig_path}/motion_update_db_fields.sh %f;${exec_str}"
                else
                    echo "on_movie_end ${exec_str}"
                fi
            else
                # error
                write_to_config_log -error "Parameter MOTION_${tidx}_ON_MOVIEEND='${m_on_movieend}'"
                write_to_config_log -error -ff "has been set but the file doesn't exist or has no execute rights!"
            fi
        fi

        # Command to be executed when a picture (.ppm|.jpg) is saved (default: none)
        # To give the filename as an argument to a command append it with %f

        if [ "${m_on_picturesave}" != "" -a "${m_on_picturesave}" != "${MOTION_DEF_ON_PICTURESAVE}" ]
        then
            exec_str=`add_script_path "${m_on_picturesave}"`

            if `check_script_exists_and_executable "${exec_str}"`
            then
                if [ "${MOTION_SQL_ENABLED}" = "yes" ]
                then
                    # update file size in sql database
                    echo "on_picture_save ${motionconfig_path}/motion_update_db_fields.sh %f;${exec_str}"
                else
                    echo "on_picture_save ${exec_str}"
                fi
            else
                # error
                write_to_config_log -error "Parameter MOTION_${tidx}_ON_PICTURESAVE='${m_on_picturesave}'"
                write_to_config_log -error -ff "has been set but the file doesn't exist or has no execute rights!"
            fi
        fi

    } > ${generate_threadconf}${tidx}.conf
}

#========================================================================================
# main
#========================================================================================

if [ "$1" = "-noconfirm" ]
then
    noconfirm=noconfirm
    shift
fi

# set motion picture/movie target path
MOTION_DEF_TARGET_PATH=${APACHE2_DOCUMENT_ROOT}/motion

# set motion web files target path
case ${MOTION_CONTROL_WEBGUI} in
    'motionbrowser')
        MOTION_WEB_TARGET_PATH=${APACHE2_DOCUMENT_ROOT}/motionbrowser
        ;;
    'motioncgi')
        MOTION_WEB_TARGET_PATH=${APACHE2_DOCUMENT_ROOT}/motion-cgi
        ;;
    *) # don't install web-gui
        ;;
esac

case "$1" in
    create-sql-db)
        create_sql_db_and_table 'force'
        ;;
    delete-sql-db)
        remove_sql_db_and_table
        ;;
    force-web-update)
        if check_active_config
        then
            extract_webfiles "${MOTION_CONTROL_WEBGUI}" force
        fi
        ;;
    *)
        # generate all configuration files
        mecho
        mecho "version: ${motion_version}"

        write_to_config_log -header

        if check_active_config
        then
            # check for installed and activated mysql
            check_installed_mysql

            MYSQL_INSTALLED=$?

            # check sql settings
            create_sql_db_and_table

            # check if the automatic database backup has been configured
            check_sql_backup

            # create target directory
            extract_webfiles "${MOTION_CONTROL_WEBGUI}"

            # create default configuration
            create_motion_config

            # create thread configuration
            rm -f ${generate_threadconf}*.conf

            idx=1
            while [ ${idx} -le ${MOTION_N} ]
            do
                eval active='$MOTION_'${idx}'_ACTIVE'

                if [ "${active}" = "yes" ]
                then
                    create_motion_thread_config ${idx}
                fi

                idx=`expr ${idx} + 1`
            done

            # create web configuration
            case ${MOTION_CONTROL_WEBGUI} in
                'motionbrowser')
                    if [ "${MOTION_SQL_ENABLED}" = "yes" ]
                    then
                        create_motionbrowser_conf
                        remove_webfiles 'motioncgi'
                    else
                        write_to_config_log -error "No configuration has been generated because parameter"
                        write_to_config_log -error -ff "MOTION_SQL_ENABLED='yes' hasn't been set."
                    fi
                    ;;

                'motioncgi')
                    create_motion_cgi_conf
                    remove_webfiles 'motionbrowser'
                    ;;

                *) # don't install any web-gui
                    remove_webfiles 'both'
                    ;;
            esac

            create_motion_wgetrc_conf
        else
            # remove configuration files
            rm -f ${generate_motionconf}
            rm -f ${generate_threadconf}*.conf

            # error
            write_to_config_log -error "No configuration has been generated because parameter"
            write_to_config_log -error -ff "START_MOTION='yes' hasn't been set or value of parameter"
            write_to_config_log -error -ff "MOTION_N='${MOTION_N}' isn't greater or equal 1!"
        fi

        compress_config_log

        if [ "${noconfirm}" != "noconfirm" ]
        then
            display_config_log
        fi

        mecho "finished."
        ;;
esac

#========================================================================================
# end
#========================================================================================
